# -*- coding: utf-8 -*-
#!/usr/bin/env python
#pylint: disable=locally-disabled,line-too-long,attribute-defined-outside-init
#pylint: disable=locally-disabled,too-many-public-methods
#
# Copyright 2015-2023 BigML
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.


""" Testing prediction creation

"""


from bigmler.tests.world import world, common_setup_module, \
    common_teardown_module, show_method, show_doc


import bigmler.tests.basic_tst_prediction_steps as test_pred


def setup_module():
    """Setup for the module

    """
    common_setup_module()
    test = TestPrediction()
    test.bigml = {"method": "setup_scenario02"}
    test.setup_scenario02()
    test.bigml = {"method": "setup_scenario06"}
    test.setup_scenario06()

def teardown_module():
    """Teardown for the module

    """
    common_teardown_module()


class TestPrediction:
    """Testing supervised predictions"""

    def setup_method(self, method):
        """
            Debug information
        """
        self.bigml = {}
        self.bigml["method"] = method.__name__
        print("\n-------------------\nTests in: %s\n" % __name__)

    def teardown_method(self):
        """Calling generic teardown for every method

        """
        world.clear_paths()
        print("\nEnd of tests in: %s\n-------------------\n" % __name__)
        self.bigml = {}

    def test_scenario01(self):
        """
        Scenario: Successfully building test predictions from start with no headers:
            Given I create BigML resources uploading train "<data>" file with no headers to test "<test>" with no headers and log predictions in "<output>"
            And I check that the source has been created
            And I check that the dataset has been created
            And I check that the model has been created
            And I check that the predictions are ready
            Then the local prediction file is like "<predictions_file>"
        """
        headers = ["data", "test", "output", "predictions_file"]
        examples = [
            ['data/iris_nh.csv', 'data/test_iris_nh.csv',
             'scenario1_nh/predictions.csv',
             'check_files/predictions_iris.csv']]
        show_doc(self.test_scenario01, examples)
        for example in examples:
            example = dict(zip(headers, example))
            show_method(self, self.bigml["method"], example)
            test_pred.i_create_all_resources_with_no_headers(
                self, example["data"], example["test"], example["output"])
            test_pred.i_check_create_source(self)
            test_pred.i_check_create_dataset(self, suffix=None)
            test_pred.i_check_create_model(self)
            test_pred.i_check_create_predictions(self)
            test_pred.i_check_predictions(self, example["predictions_file"])

    def setup_scenario02(self):
        """
        Scenario: Successfully building test predictions from start:
            Given I create BigML resources uploading train "<data>" file to test "<test>" and log predictions in "<output>"
            And I check that the source has been created
            And I check that the dataset has been created
            And I check that the model has been created
            And I check that the predictions are ready
            Then the local prediction file is like "<predictions_file>"
        """
        headers = ["data", "test", "output", "predictions_file"]
        examples = [
            ['data/grades.csv', 'data/test_grades.csv',
             'scenario1_r/predictions.csv',
             'check_files/predictions_grades.csv'],
            ['data/iris.csv', 'data/test_iris.csv',
             'scenario1/predictions.csv', 'check_files/predictions_iris.csv']]
        show_doc(self.setup_scenario02, examples)
        for example in examples:
            example = dict(zip(headers, example))
            show_method(self, self.bigml["method"], example)
            test_pred.i_create_all_resources(
                self, example["data"], example["test"], example["output"])
            test_pred.i_check_create_source(self)
            test_pred.i_check_create_dataset(self, suffix=None)
            test_pred.i_check_create_model(self)
            test_pred.i_check_create_predictions(self)
            test_pred.i_check_predictions(self, example["predictions_file"])

    def test_scenario03(self):
        """
        Scenario: Successfully building test predictions from source
            Given I have previously executed "<scenario>" or reproduce it with arguments <kwargs>
            And I create BigML resources using source to test "<test>" and log predictions in "<output>"
            And I check that the dataset has been created
            And I check that the model has been created
            And I check that the predictions are ready
            Then the local prediction file is like "<predictions_file>"
        """
        headers = ["scenario", "kwargs", "test", "output", "predictions_file"]
        examples = [
            ['scenario1', '{"data": "data/iris.csv",' +
             ' "output": "scenario1/predictions.csv",' +
             ' "test": "data/test_iris.csv"}', 'data/test_iris.csv',
             'scenario2/predictions.csv', 'check_files/predictions_iris.csv']]
        show_doc(self.test_scenario03, examples)
        for example in examples:
            example = dict(zip(headers, example))
            show_method(self, self.bigml["method"], example)
            test_pred.i_have_previous_scenario_or_reproduce_it(
                self, example["scenario"], example["kwargs"])
            test_pred.i_create_resources_from_source(
                self, None, test=example["test"], output=example["output"])
            test_pred.i_check_create_dataset(self, suffix=None)
            test_pred.i_check_create_model(self)
            test_pred.i_check_create_predictions(self)
            test_pred.i_check_predictions(self, example["predictions_file"])

    def test_scenario04(self):
        """
        Scenario: Successfully building test predictions from dataset
            Given I have previously executed "<scenario>" or reproduce it with arguments <kwargs>
            And I create BigML resources using dataset to test "<test>" and log predictions in "<output>"
            And I check that the model has been created
            And I check that the predictions are ready
            Then the local prediction file is like "<predictions_file>"
        """
        headers = ["scenario", "kwargs", "test", "output", "predictions_file"]
        examples = [
            ['scenario1', '{"data": "data/iris.csv",' +
             ' "output": "scenario1/predictions.csv",' +
             ' "test": "data/test_iris.csv"}', 'data/test_iris.csv',
             'scenario3/predictions.csv', 'check_files/predictions_iris.csv']]
        show_doc(self.test_scenario04, examples)
        for example in examples:
            example = dict(zip(headers, example))
            show_method(self, self.bigml["method"], example)
            test_pred.i_have_previous_scenario_or_reproduce_it(
                self, example["scenario"], example["kwargs"])
            test_pred.i_create_resources_from_dataset(
                self, None, test=example["test"], output=example["output"])
            test_pred.i_check_create_model(self)
            test_pred.i_check_create_predictions(self)
            test_pred.i_check_predictions(self, example["predictions_file"])

    def test_scenario05(self):
        """
        Scenario: Successfully building test predictions from model
            Given I have previously executed "<scenario>" or reproduce it with arguments <kwargs>
            And I create BigML resources using model to test "<test>" and log predictions in "<output>"
            And I check that the predictions are ready
            Then the local prediction file is like "<predictions_file>"
        """
        headers = ["scenario", "kwargs", "test", "output", "predictions_file"]
        examples = [
            ['scenario1', '{"data": "data/iris.csv",' +
             ' "output": "scenario1/predictions.csv",' +
             ' "test": "data/test_iris.csv"}', 'data/test_iris.csv',
             'scenario4/predictions.csv', 'check_files/predictions_iris.csv']]
        show_doc(self.test_scenario05, examples)
        for example in examples:
            example = dict(zip(headers, example))
            show_method(self, self.bigml["method"], example)
            test_pred.i_have_previous_scenario_or_reproduce_it(
                self, example["scenario"], example["kwargs"])
            test_pred.i_create_resources_from_model(
                self, test=example["test"], output=example["output"])
            test_pred.i_check_create_predictions(self)
            test_pred.i_check_predictions(self, example["predictions_file"])

    def setup_scenario06(self):
        """
        Scenario: Successfully building test predictions from ensemble
            Given I have previously executed "<scenario>" or reproduce it with arguments <kwargs>
            And I create BigML resources using ensemble of <number_of_models> models to test "<test>" and log predictions in "<output>"
            And I check that the ensemble has been created
            And I check that the predictions are ready
            Then the local prediction file is like "<predictions_file>"
        """
        headers = ["scenario", "kwargs", "number_of_models", "test", "output",
                   "predictions_file"]
        examples = [
            ['scenario1', '{"data": "data/iris.csv",' +
             ' "output": "scenario1/predictions.csv",' +
             ' "test": "data/test_iris.csv"}', '10', 'data/test_iris.csv',
             'scenario5/predictions.csv', 'check_files/predictions_iris.csv']]
        show_doc(self.setup_scenario06, examples)
        for example in examples:
            example = dict(zip(headers, example))
            show_method(self, self.bigml["method"], example)
            test_pred.i_have_previous_scenario_or_reproduce_it(
                self, example["scenario"], example["kwargs"])
            test_pred.i_create_resources_from_ensemble(
                self, number_of_models=example["number_of_models"],
                test=example["test"],
                output=example["output"])
            test_pred.i_check_create_ensemble(self)
            test_pred.i_check_create_predictions(self)
            test_pred.i_check_predictions(self, example["predictions_file"])

    def test_scenario07(self):
        """
        Scenario: Successfully building test predictions from models file
            Given I have previously executed "<scenario>" or reproduce it with arguments <kwargs>
            And I have previously executed "<scenario2>" or reproduce it with arguments <kwargs2>
            And I create BigML resources using models in file "<models_file>" to test "<test>" and log predictions in "<output>"
            And I check that the predictions are ready
            Then the local prediction file is like "<predictions_file>"
        """
        print(self.test_scenario07.__doc__)
        headers = ["scenario", "kwargs", "scenario2", "kwargs2", "models_file",
                   "test", "output", "predictions_file"]
        examples = [
            ['scenario1', '{"data": "data/iris.csv",' +
             ' "output": "scenario1/predictions.csv",' +
             ' "test": "data/test_iris.csv"}',
             'scenario5', '{"number_of_models": 10,' +
             ' "test": "data/test_iris.csv",' +
             ' "output": "scenario5/predictions.csv"}',
             'scenario5/models', 'data/test_iris.csv',
             'scenario6/predictions.csv', 'check_files/predictions_iris.csv']]
        show_doc(self.test_scenario07, examples)
        for example in examples:
            example = dict(zip(headers, example))
            show_method(self, self.bigml["method"], example)
            test_pred.i_have_previous_scenario_or_reproduce_it(
                self, example["scenario"], example["kwargs"])
            test_pred.i_have_previous_scenario_or_reproduce_it(
                self, example["scenario2"], example["kwargs2"])
            test_pred.i_create_resources_from_models_file(
                self, multi_label=None, models_file=example["models_file"],
                test=example["test"], output=example["output"])
            test_pred.i_check_create_predictions(self)
            test_pred.i_check_predictions(self, example["predictions_file"])

    def test_scenario08(self):
        """
        Scenario: Successfully building test predictions from dataset file
            Given I have previously executed "<scenario>" or reproduce it with arguments <kwargs>
            And I create BigML resources using dataset in file "<dataset_file>" to test "<test>" and log predictions in "<output>"
            And I check that the model has been created
            And I check that the predictions are ready
            Then the local prediction file is like "<predictions_file>"
        """
        headers = ["scenario", "kwargs", "dataset_file", "test", "output",
                   "predictions_file"]
        examples = [
            ['scenario1', '{"data": "data/iris.csv",' +
             ' "output": "scenario1/predictions.csv",' +
             ' "test": "data/test_iris.csv"}', 'scenario1/dataset',
             'data/test_iris.csv', 'scenario7/predictions.csv',
             'check_files/predictions_iris.csv']]
        show_doc(self.test_scenario08, examples)
        for example in examples:
            example = dict(zip(headers, example))
            show_method(self, self.bigml["method"], example)
            test_pred.i_have_previous_scenario_or_reproduce_it(
                self, example["scenario"], example["kwargs"])
            test_pred.i_create_resources_from_dataset_file(
                self, dataset_file=example["dataset_file"],
                test=example["test"], output=example["output"])
            test_pred.i_check_create_model(self)
            test_pred.i_check_create_predictions(self)
            test_pred.i_check_predictions(self, example["predictions_file"])

    def test_scenario09(self):
        """
        Scenario: Successfully combining test predictions from existing directories
            Given I have previously executed "<scenario>" or reproduce it with arguments <kwargs>
            And I have previously executed "<scenario2>" or reproduce it with arguments <kwargs2>
            Given I combine BigML predictions files in "<directory1>" and "<directory2>" into "<output>"
            Then the local prediction file is like "<predictions_file>"
        """
        headers =["scenario", "kwargs", "scenario2", "kwargs2", "directory1",
                  "directory2", "output", "predictions_file"]
        examples = [
            ['scenario1', '{"data": "data/iris.csv",' +
             ' "output": "scenario1/predictions.csv",' +
             ' "test": "data/test_iris.csv"}',
             'scenario5', '{"number_of_models": 10,' +
             ' "test": "data/test_iris.csv",' +
             ' "output": "scenario5/predictions.csv"}',
             'scenario1', 'scenario5', 'scenario8/predictions.csv',
             'check_files/predictions_iris.csv']]
        show_doc(self.test_scenario09, examples)
        for example in examples:
            example = dict(zip(headers, example))
            show_method(self, self.bigml["method"], example)
            test_pred.i_have_previous_scenario_or_reproduce_it(
                self, example["scenario"], example["kwargs"])
            test_pred.i_have_previous_scenario_or_reproduce_it(
                self, example["scenario2"], example["kwargs2"])
            test_pred.i_find_predictions_files(
                self, directory1=example["directory1"],
                directory2=example["directory2"],
                output=example["output"])
            test_pred.i_check_predictions(self, example["predictions_file"])

    def test_scenario10(self):
        """
        Scenario: Successfully combining test predictions from existing directories
            Given I have previously executed "<scenario>" or reproduce it with arguments <kwargs>
            And I have previously executed "<scenario2>" or reproduce it with arguments <kwargs2>
            And I combine BigML predictions files in "<directory1>" and "<directory2>" into "<output>" with method "<method>"
            Then the local prediction file is like "<predictions_file>"
        """
        headers = ["scenario", "kwargs", "scenario2", "kwargs2", "directory1",
                   "directory2", "output", "predictions_file", "method"]
        examples = [
            ['scenario1', '{"data": "data/iris.csv",' +
             ' "output": "scenario1/predictions.csv",' +
             ' "test": "data/test_iris.csv"}',
             'scenario5', '{"number_of_models": 10,' +
             ' "output": "scenario5/predictions.csv",' +
             ' "test": "data/test_iris.csv"}',
             'scenario1', 'scenario5', 'scenario9/predictions_c.csv',
             'check_files/predictions_iris.csv', '"confidence weighted"'],
            ['scenario1', '{"data": "data/iris.csv",' +
             ' "output": "scenario1/predictions.csv",' +
             ' "test": "data/test_iris.csv"}',
             'scenario5', '{"number_of_models": 10,' +
             ' "output": "scenario5/predictions.csv",' +
             ' "test": "data/test_iris.csv"}',
             'scenario1', 'scenario5', 'scenario9/predictions_p.csv',
             'check_files/predictions_iris_p.csv', '"probability weighted"'],
            ['scenario1_r', '{"data": "data/grades.csv",' +
              ' "output": "scenario1_r/predictions.csv",' +
              ' "test": "data/test_grades.csv"}',
              'scenario1_r', '{"data": "data/grades.csv",' +
              ' "output": "scenario1_r/predictions.csv",' +
              ' "test": "data/test_grades.csv"}',
             'scenario1_r', 'scenario1_r', 'scenario10/predictions_c.csv',
             'check_files/predictions_grades.csv', '"confidence weighted"'],
            ['scenario1_r', '{"data": "data/grades.csv",' +
             ' "output": "scenario1_r/predictions.csv",' +
             ' "test": "data/test_grades.csv"}',
             'scenario1_r', '{"data": "data/grades.csv",' +
             ' "output": "scenario1_r/predictions.csv",' +
             ' "test": "data/test_grades.csv"}',
             'scenario1_r', 'scenario1_r', 'scenario10/predictions_p.csv',
             'check_files/predictions_grades_p.csv', '"probability weighted"']]
        show_doc(self.test_scenario10, examples)
        for example in examples:
            example = dict(zip(headers, example))
            show_method(self, self.bigml["method"], example)
            test_pred.i_have_previous_scenario_or_reproduce_it(
                self, example["scenario"], example["kwargs"])
            test_pred.i_have_previous_scenario_or_reproduce_it(
                self, example["scenario2"], example["kwargs2"])
            test_pred.i_find_predictions_files_with_method(
                self, directory1=example["directory1"],
                directory2=example["directory2"],
                output=example["output"], method=example["method"])
            test_pred.i_check_predictions(self, example["predictions_file"])

    def test_scenario11(self):
        """
        Scenario: Successfully building test predictions from dataset specifying objective field and model fields
            Given I have previously executed "<scenario>" or reproduce it with arguments <kwargs>
            And I create BigML resources using dataset, objective field <objective> and model fields <fields> to test "<test>" and log predictions in "<output>"
            And I check that the model has been created
            And I check that the predictions are ready
            Then the local prediction file is like "<predictions_file>"
        """
        headers = ["scenario", "kwargs", "test", "output", "predictions_file",
                   "objective", "fields"]
        examples = [
            ['scenario1', '{"data": "data/iris.csv",' +
             ' "output": "scenario1/predictions.csv",' +
             ' "test": "data/test_iris.csv"}', 'data/test_iris.csv',
             'scenario11/predictions.csv',
             'check_files/predictions_iris_b.csv', '0',
             '"petal length","petal width"']]
        show_doc(self.test_scenario11, examples)
        for example in examples:
            example = dict(zip(headers, example))
            show_method(self, self.bigml["method"], example)
            test_pred.i_have_previous_scenario_or_reproduce_it(
                self, example["scenario"], example["kwargs"])
            test_pred.i_create_resources_from_dataset_objective_model(
                self, objective=example["objective"], fields=example["fields"],
                test=example["test"], output=example["output"])
            test_pred.i_check_create_model(self)
            test_pred.i_check_create_predictions(self)
            test_pred.i_check_predictions(self, example["predictions_file"])

    def test_scenario12(self):
        """
        Scenario: Successfully building cross-validation from dataset
            Given I have previously executed "<scenario>" or reproduce it with arguments <kwargs>
            And I create a BigML cross-validation with rate <rate> using the dataset in file "<dataset_file>" and log results in "<output>"
            And I check that the models have been created
            And I check that the evaluations have been created
            Then the cross-validation json model info is like the one in "<cv_file>"
        """
        headers = ["scenario", "kwargs", "rate", "dataset_file", "output",
                   "cv_file"]
        examples = [
            ['scenario1', '{"data": "data/iris.csv",' +
             ' "output": "scenario1/predictions.csv",' +
             ' "test": "data/test_iris.csv"}', '0.05', 'scenario1/dataset',
             'scenario12/cross-validation',
             'check_files/cross_validation.json']]
        show_doc(self.test_scenario12, examples)
        for example in examples:
            example = dict(zip(headers, example))
            show_method(self, self.bigml["method"], example)
            test_pred.i_have_previous_scenario_or_reproduce_it(
                self, example["scenario"], example["kwargs"])
            test_pred.i_create_cross_validation_from_dataset(
                self, rate=example["rate"],
                dataset_file=example["dataset_file"],
                output=example["output"])
            test_pred.i_check_create_models(self)
            test_pred.i_check_create_evaluations(
                self, number_of_evaluations=None)
            test_pred.i_check_cross_validation(
                self, example["cv_file"])

    def test_scenario13(self):
        """
        Scenario: Successfully building a source with a given locale and storing its result
            Given I create a BigML source from file "<data>" with locale "<locale>", field attributes "<field_attributes>" and types file "<types>" storing results in "<output>"
            Then I check that the stored source file exists
            And the locale of the source is "<bigml_locale>"
            And the type of field "<field_id>" is "<type>"
            And the label of field "<field_id>" is "<label>"
        """
        headers = ["data", "locale", "field_attributes", "types", "output",
                   "bigml_locale", "field_id", "type", "label"]
        examples = [
            ['data/iris.csv', 'es_ES.UTF-8', 'data/field_attributes.txt', 'data/types.txt', 'scenario13/store_file', 'es_ES', '000004', 'text', 'species label']]
        show_doc(self.test_scenario13, examples)
        for example in examples:
            example = dict(zip(headers, example))
            show_method(self, self.bigml["method"], example)
            test_pred.i_create_source_with_locale(
                self, data=example["data"], locale=example["locale"],
                field_attributes=example["field_attributes"],
                types=example["types"], output=example["output"])
            test_pred.i_check_stored_source(self)
            test_pred.i_check_source_locale(self, example["bigml_locale"])
            test_pred.i_check_source_type(
                self, example["field_id"], example["type"])
            test_pred.i_check_source_label(
                self, example["field_id"], example["label"])

    def test_scenario14(self):
        """
        Scenario: Successfully building test predictions from start with user-given separator:
            Given I create BigML resources uploading train "<data>" file to test "<test>" and log predictions in "<output>" with "<separator>" as test field separator
            And I check that the source has been created
            And I check that the dataset has been created
            And I check that the model has been created
            And I check that the predictions are ready
            Then the local prediction file is like "<predictions_file>"
        """
        headers = ["data", "test", "separator", "output", "predictions_file"]
        examples = [
            ['data/iris.csv', 'data/test_iris.tsv', '"\t"',
             'scenario14/predictions.csv', 'check_files/predictions_iris.csv']]
        show_doc(self.test_scenario14, examples)
        for example in examples:
            example = dict(zip(headers, example))
            show_method(self, self.bigml["method"], example)
            test_pred.i_create_all_resources_with_separator(
                self, data=example["data"], test=example["test"],
                output=example["output"], separator=example["separator"])
            test_pred.i_check_create_source(self)
            test_pred.i_check_create_dataset(self, suffix=None)
            test_pred.i_check_create_model(self)
            test_pred.i_check_create_predictions(self)
            test_pred.i_check_predictions(self, example["predictions_file"])

    def test_scenario15(self):
        """
        Scenario: Successfully building test predictions from start with different prediction file format:
            Given I create BigML resources uploading train "<data>" file to test "<test>" and log predictions in "<output>" with prediction options "<options>"
            And I check that the source has been created
            And I check that the dataset has been created
            And I check that the model has been created
            And I check that the predictions are ready
            Then the local prediction file is like "<predictions_file>"
        """
        headers = ["data", "test", "output", "options", "predictions_file"]
        examples = [
            ['data/iris.csv', 'data/test_iris.csv',
             'scenario15/predictions.csv',
             '--prediction-header --prediction-fields \'petal length,' +
             'petal width\' --prediction-info full',
             'check_files/predictions_iris_h.csv']]
        show_doc(self.test_scenario15, examples)
        for example in examples:
            example = dict(zip(headers, example))
            show_method(self, self.bigml["method"], example)
            test_pred.i_create_all_resources_with_options(
                self, data=example["data"], test=example["test"],
                output=example["output"], options=example["options"])
            test_pred.i_check_create_source(self)
            test_pred.i_check_create_dataset(self, suffix=None)
            test_pred.i_check_create_model(self)
            test_pred.i_check_create_predictions(self)
            test_pred.i_check_predictions(self, example["predictions_file"])

    def test_scenario16(self):
        """
        Scenario: Successfully building threshold test predictions from ensemble
            Given I have previously executed "<scenario>" or reproduce it with arguments <kwargs>
            And I create BigML resources using ensemble of <number_of_models> models with replacement to test "<test>" and log predictions in "<output>"
            And I check that the ensemble has been created
            And I check that the predictions are ready
            And I create BigML resources using the previous ensemble with different thresholds to test "<test>" and log predictions in "<output2>" and "<output3>"
            Then local predictions for different thresholds in "<output2>" and "<output3>" are different
        """
        headers = ["scenario", "kwargs", "number_of_models", "test", "output",
                   "output2", "output3"]
        examples = [
            ['scenario1', '{"data": "data/iris.csv",' +
             ' "output": "scenario1/predictions.csv",' +
             ' "test": "data/test_iris.csv"}', '10', 'data/test_iris.csv',
             'scenario16/predictions.csv', 'scenario16/predictions2.csv',
             'scenario16/predictions3.csv']]
        show_doc(self.test_scenario16, examples)
        for example in examples:
            example = dict(zip(headers, example))
            show_method(self, self.bigml["method"], example)
            test_pred.i_have_previous_scenario_or_reproduce_it(
                self, example["scenario"], example["kwargs"])
            test_pred.i_create_resources_from_ensemble_with_replacement(
                self, number_of_models=example["number_of_models"],
                test=example["test"], output=example["output"])
            test_pred.i_check_create_ensemble(self)
            test_pred.i_check_create_predictions(self)
            test_pred.i_create_resources_from_ensemble_with_threshold(
                self, test=example["test"], output2=example["output2"],
                output3=example["output3"])
            test_pred.i_check_predictions_with_different_thresholds(
                self, example["output2"], example["output3"])

    def test_scenario17(self):
        """
        Scenario: Successfully building test predictions from local model
            Given I have previously executed "<scenario>" or reproduce it with arguments <kwargs>
            And I create BigML resources using local model in "<scenario>" to test "<test>" and log predictions in "<output>"
            And I check that the predictions are ready
            Then the local prediction file is like "<predictions_file>"
        """
        headers = ["scenario", "kwargs", "test", "output", "predictions_file"]
        examples = [
            ['scenario1', '{"data": "data/iris.csv",' +
             ' "output": "scenario1/predictions.csv",' +
             ' "test": "data/test_iris.csv"}', 'data/test_iris.csv',
             'scenario17/predictions.csv', 'check_files/predictions_iris.csv']]
        show_doc(self.test_scenario17, examples)
        for example in examples:
            example = dict(zip(headers, example))
            show_method(self, self.bigml["method"], example)
            test_pred.i_have_previous_scenario_or_reproduce_it(
                self, example["scenario"], example["kwargs"])
            test_pred.i_create_resources_from_local_model(
                self, directory=example["scenario"], test=example["test"],
                output=example["output"])
            test_pred.i_check_create_predictions(self)
            test_pred.i_check_predictions(self, example["predictions_file"])

    def test_scenario18(self):
        """
        Scenario: Successfully building test predictions from ensemble
            Given I have previously executed "<scenario>" or reproduce it with arguments <kwargs>
            Given I have previously executed "<scenario2>" or reproduce it with arguments <kwargs2>
            And I create BigML resources using local ensemble of <number_of_models> models in "<scenario2>" to test "<test>" and log predictions in "<output>"
            And I check that the predictions are ready
            Then the local prediction file is like "<predictions_file>"
        """
        headers = ["scenario", "kwargs", "scenario2", "kwargs2",
                   "number_of_models", "directory", "test",
                   "output", "predictions_file"]
        examples = [
            ['scenario1', '{"data": "data/iris.csv",' +
             ' "output": "scenario1/predictions.csv",' +
             ' "test": "data/test_iris.csv"}',
             'scenario5', '{"number_of_models": 10,' +
             ' "output": "scenario5/predictions.csv",' +
             ' "test": "data/test_iris.csv"}',
             '10', 'scenario5', 'data/test_iris.csv',
             'scenario18/predictions.csv', 'check_files/predictions_iris.csv']]
        show_doc(self.test_scenario18, examples)
        for example in examples:
            example = dict(zip(headers, example))
            show_method(self, self.bigml["method"], example)
            test_pred.i_have_previous_scenario_or_reproduce_it(
                self, example["scenario"], example["kwargs"])
            test_pred.i_have_previous_scenario_or_reproduce_it(
                self, example["scenario2"], example["kwargs2"])
            test_pred.i_create_resources_from_local_ensemble(
                self, number_of_models=example["number_of_models"],
                directory=example["directory"], test=example["test"],
                output=example["output"])
            test_pred.i_check_create_predictions(self)
            test_pred.i_check_predictions(self, example["predictions_file"])

    def test_scenario19(self):
        """
            Scenario: Successfully building test predictions from start using median:
                Given I create BigML resources uploading train "<data>" file using the median to test "<test>" and log predictions in "<output>"
                And I check that the source has been created
                And I check that the dataset has been created
                And I check that the model has been created
                And I check that the predictions are ready
                Then the local prediction file is like "<predictions_file>"
        """
        headers = ["data", "test", "output", "predictions_file"]
        examples = [
            ['data/grades.csv', 'data/test_grades.csv',
             'scenario19/predictions.csv',
             'check_files/predictions_grades_median.csv']]
        show_doc(self.test_scenario19, examples)
        for example in examples:
            example = dict(zip(headers, example))
            show_method(self, self.bigml["method"], example)
            test_pred.i_create_all_resources_with_median(
                self, data=example["data"], test=example["test"],
                output=example["output"])
            test_pred.i_check_create_source(self)
            test_pred.i_check_create_dataset(self, suffix=None)
            test_pred.i_check_create_model(self)
            test_pred.i_check_create_predictions(self)
            test_pred.i_check_predictions(self, example["predictions_file"])

    def test_scenario20(self):
        """
        Scenario: Successfully building test predictions using median from ensemble
                Given I create BigML resources from "<data>" using ensemble of <number_of_models> models to test "<test>" using median and log predictions in "<output>"
                And I check that the source has been created
                And I check that the dataset has been created
                And I check that the ensemble has been created
                And I check that the models in the ensembles have been created
                And I check that the predictions are ready
                Then the local prediction file is like "<predictions_file>"
        """
        headers = ["data", "number_of_models", "test", "output",
                   "predictions_file"]
        examples = [
            ['data/grades.csv', '5', 'data/test_grades.csv',
             'scenario20/predictions.csv',
             'check_files/predictions_grades_median_e.csv']]
        show_doc(self.test_scenario20, examples)
        for example in examples:
            example = dict(zip(headers, example))
            show_method(self, self.bigml["method"], example)
            test_pred.i_create_resources_from_ensemble_using_median(
                self, data=example["data"],
                number_of_models=example["number_of_models"],
                test=example["test"], output=example["output"])
            test_pred.i_check_create_source(self)
            test_pred.i_check_create_dataset(self, suffix=None)
            test_pred.i_check_create_ensemble(self)
            test_pred.i_check_create_models_in_ensembles(self, in_ensemble=True)
            test_pred.i_check_create_predictions(self)
            test_pred.i_check_predictions(self, example["predictions_file"])

    def test_scenario21(self):
        """
        Scenario 1: Successfully building test predictions from ensemble
                And I create BigML resources from "<data>" using ensemble of <number_of_models> models to test "<test>" and log predictions in "<output>"
                And I check that the source has been created
                And I check that the dataset has been created
                And I check that the ensemble has been created
                And I check that the models in the ensembles have been created
                And I check that the predictions are ready
                Then the local prediction file is like "<predictions_file>"

                Examples:
                |data               | number_of_models | test                      | output                         | predictions_file                         |
        """
        headers = ["data", "number_of_models", "test", "output",
                   "predictions_file"]
        examples = [
            ['data/grades.csv', '5', 'data/test_grades.csv',
             'scenario21/predictions.csv',
             'check_files/predictions_grades_e.csv']]
        show_doc(self.test_scenario21, examples)
        for example in examples:
            example = dict(zip(headers, example))
            show_method(self, self.bigml["method"], example)
            test_pred.i_create_resources_in_prod_from_ensemble(
                self, data=example["data"],
                number_of_models=example["number_of_models"],
                test=example["test"], output=example["output"])
            test_pred.i_check_create_source(self)
            test_pred.i_check_create_dataset(self, suffix=None)
            test_pred.i_check_create_ensemble(self)
            test_pred.i_check_create_models_in_ensembles(
                self, in_ensemble=True)
            test_pred.i_check_create_predictions(self)
            test_pred.i_check_predictions(self, example["predictions_file"])

    def test_scenario22(self):
        """
        Scenario: Successfully building threshold test predictions remotely from ensemble
            Given I have previously executed "<scenario>" or reproduce it with arguments <kwargs>
            And I create BigML resources using ensemble of <number_of_models> models with replacement to test "<test>" and log predictions in "<output>"
            And I check that the ensemble has been created
            And I check that the predictions are ready
            And I create BigML resources using the previous ensemble with different thresholds "<threshold_class>" to remotely test "<test>" and log predictions in "<output2>" and "<output3>"
            Then local predictions for different thresholds in "<output2>" and "<output3>" are different
        """
        headers = ["scenario", "kwargs", "number_of_models", "test", "output",
                   "output2", "output3", "threshold_class"]
        examples = [
            ['scenario1', '{"data": "data/iris.csv",' +
             ' "output": "scenario1/predictions.csv",' +
             ' "test": "data/test_iris.csv"}', '10', 'data/test_iris.csv',
             'scenario22/predictions.csv', 'scenario22/predictions2.csv',
             'scenario22/predictions3.csv', 'Iris-virginica']]
        show_doc(self.test_scenario22, examples)
        for example in examples:
            example = dict(zip(headers, example))
            show_method(self, self.bigml["method"], example)
            test_pred.i_have_previous_scenario_or_reproduce_it(
                self, example["scenario"], example["kwargs"])
            test_pred.i_create_resources_from_ensemble_with_replacement(
                self, number_of_models=example["number_of_models"],
                test=example["test"], output=example["output"])
            test_pred.i_check_create_ensemble(self)
            test_pred.i_check_create_predictions(self)
            test_pred.i_create_resources_from_ensemble_with_threshold_rem(
                self, test=example["test"], output2=example["output2"],
                output3=example["output3"],
                threshold_class=example["threshold_class"])
            test_pred.i_check_predictions_with_different_thresholds(
                self, example["output2"], example["output3"])

    def test_scenario23(self):
        """
        Scenario: Successfully building test predictions from boosted ensemble
            Given I have previously executed "<scenario>" or reproduce it with arguments <kwargs>
            And I create BigML resources using boosted ensemble in <iterations> iterations to test "<test>" and log predictions in "<output>"
            And I check that the ensemble has been created
            And I check that the predictions are ready
            Then the local prediction file is like "<predictions_file>"
        """
        headers = ["scenario", "kwargs", "iterations", "test", "output",
                   "predictions_file"]
        examples = [
            ['scenario1', '{"data": "data/iris.csv", "output": "scenario1/predictions.csv", "test": "data/test_iris.csv"}', '10', 'data/test_iris.csv', 'scenario23/predictions.csv', 'check_files/predictions_iris_boost.csv']]
        show_doc(self.test_scenario23, examples)
        for example in examples:
            example = dict(zip(headers, example))
            show_method(self, self.bigml["method"], example)
            test_pred.i_have_previous_scenario_or_reproduce_it(
                self, example["scenario"], example["kwargs"])
            test_pred.i_create_resources_from_boosted_ensemble(
                self, iterations=example["iterations"], test=example["test"],
                output=example["output"])
            test_pred.i_check_create_ensemble(self)
            test_pred.i_check_create_predictions(self)
            test_pred.i_check_predictions(self, example["predictions_file"])


    def test_scenario24(self):
        """
        Scenario: Successfully  test predictions remotely from boosted ensemble
            Given I have previously executed "<scenario>" or reproduce it with arguments <kwargs>
            And I create BigML resources using boosted ensemble in <iterations> iterations to remotely test "<test>" and log predictions in "<output>"
            And I check that the ensemble has been created
            And I check that the predictions are ready
            And I check that the batch prediction is ready
            And I check that the bath predictions datset is ready
            Then the local prediction file is like "<predictions_file>"
        """
        headers = ["scenario", "kwargs", "iterations", "test", "output",
                   "predictions_file"]
        examples = [
            ['scenario1', '{"data": "data/iris.csv", "output": "scenario1/predictions.csv", "test": "data/test_iris.csv"}', '10', 'data/test_iris.csv', 'scenario24/predictions.csv', 'check_files/predictions_iris_boost.csv']]
        show_doc(self.test_scenario24, examples)
        for example in examples:
            example = dict(zip(headers, example))
            show_method(self, self.bigml["method"], example)
            test_pred.i_have_previous_scenario_or_reproduce_it(
                self, example["scenario"], example["kwargs"])
            test_pred.i_create_resources_remotely_from_boosted_ensemble(
                self, iterations=example["iterations"], test=example["test"],
                output=example["output"])
            test_pred.i_check_create_ensemble(self)
            test_pred.i_check_create_batch_prediction(self)
            test_pred.i_check_create_predictions(self)
            test_pred.i_check_predictions(self, example["predictions_file"],
                headers=True)

    def test_scenario25(self):
        """
        Scenario: Successfully building test predictions from model with operating point
            Given I have previously executed "<scenario>" or reproduce it with arguments <kwargs>
            And I create BigML resources using model with operating point "<operating_point>" to test "<test>" and log predictions in "<output>"
            And I check that the predictions are ready
            Then the local prediction file is like "<predictions_file>"
        """
        headers = ["scenario", "kwargs", "test", "output",
                   "predictions_file", "operating_point"]
        examples = [
            ['scenario1', '{"data": "data/iris.csv",' +
             ' "output": "scenario1/predictions.csv",' +
             ' "test": "data/test_iris.csv"}', 'data/test_iris.csv',
             'scenario25/predictions_p.csv',
             'check_files/predictions_iris_op_prob.csv',
             "data/operating_point_prob.json"],
            ['scenario1', '{"data": "data/iris.csv",' +
             ' "output": "scenario1/predictions.csv",' +
             ' "test": "data/test_iris.csv"}', 'data/test_iris.csv',
             'scenario25/predictions_c.csv',
             'check_files/predictions_iris_op_conf.csv',
             "data/operating_point_conf.json"]]

        show_doc(self.test_scenario25, examples)
        for example in examples:
            example = dict(zip(headers, example))
            show_method(self, self.bigml["method"], example)
            test_pred.i_have_previous_scenario_or_reproduce_it(
                self, example["scenario"], example["kwargs"])
            test_pred.i_create_resources_from_model_with_op(
                self, operating_point=example["operating_point"],
                test=example["test"], output=example["output"])
            test_pred.i_check_create_predictions(self)
            test_pred.i_check_predictions(self, example["predictions_file"])

    def test_scenario26(self):
        """
        Scenario: Successfully building test predictions from ensemble
            Given I have previously executed "<scenario>" or reproduce it with arguments <kwargs>
            Given I have previously executed "<scenario2>" or reproduce it with arguments <kwargs2>
            And I create BigML resources using local ensemble of <number_of_models> models in "<scenario2>" to test "<test>" and log predictions in "<output>"
            And I check that the predictions are ready
            Then the local prediction file is like "<predictions_file>"
        """
        headers = ["scenario", "kwargs", "scenario2", "kwargs2",
                   "number_of_models", "directory", "test", "output",
                   "predictions_file", "operating_point"]
        examples = [
            ['scenario1', '{"data": "data/iris.csv",' +
             ' "output": "scenario1/predictions.csv",' +
             ' "test": "data/test_iris.csv"}',
             'scenario5', '{"number_of_models": 10,' +
             ' "output": "scenario5/predictions.csv",' +
             ' "test": "data/test_iris.csv"}',
             '10', 'scenario5', 'data/test_iris.csv',
             'scenario26/predictions_p.csv',
             'check_files/predictions_iris_e_op_prob.csv',
             'data/operating_point_prob.json'],
            ['scenario1', '{"data": "data/iris.csv",' +
             ' "output": "scenario1/predictions.csv",' +
             ' "test": "data/test_iris.csv"}',
             'scenario5', '{"number_of_models": 10,' +
             '"output": "scenario5/predictions.csv",' +
             '"test": "data/test_iris.csv"}',
             '10', 'scenario5', 'data/test_iris.csv',
             'scenario26/predictions_c.csv',
             'check_files/predictions_iris_e_op_conf.csv',
             'data/operating_point_conf.json']]
        show_doc(self.test_scenario26, examples)
        for example in examples:
            example = dict(zip(headers, example))
            show_method(self, self.bigml["method"], example)
            test_pred.i_have_previous_scenario_or_reproduce_it(
                self, example["scenario"], example["kwargs"])
            test_pred.i_have_previous_scenario_or_reproduce_it(
                self, example["scenario2"], example["kwargs2"])
            test_pred.i_create_resources_from_local_ensemble_with_op(
                self, number_of_models=example["number_of_models"],
                directory=example["directory"],
                test=example["test"], output=example["output"],
                operating_point=example["operating_point"])
            test_pred.i_check_create_predictions(self)
            test_pred.i_check_predictions(self, example["predictions_file"])

    def test_scenario27(self):
        """
        Scenario: Successfully building test predictions from start with split field:
            Given I create BigML resources uploading train "<data>" file with split field "<split_field>" and objective "<objeciive>" and log in "<output-dir>"
            And I check that the source has been created
            And I check that the dataset has been created
            And I check that the model has been created
            And I check that the first node has <children> branches
        """
        headers = ["data", "split_field", "objective", "output_dir",
                   "children"]
        examples = [
            ['data/iris.csv', 'sepal length', '000004', 'scenario27', 3]]
        show_doc(self.test_scenario27, examples)
        for example in examples:
            example = dict(zip(headers, example))
            show_method(self, self.bigml["method"], example)
            test_pred.i_create_all_resources_with_split_field(
                self, example["data"], example["objective"],
                example["split_field"], example["output_dir"])
            test_pred.i_check_create_source(self)
            test_pred.i_check_create_dataset(self, suffix=None)
            test_pred.i_check_create_model(self)
            test_pred.i_check_first_node_children(
                self, example["children"], example["objective"])

    def test_scenario28(self):
        """
        Scenario: Successfully building test predictions from start with focus field:
            Given I create BigML resources uploading train "<data>" file with focus field "<focus_field>" and objective "<objeciive>" and log in "<output-dir>"
            And I check that the source has been created
            And I check that the dataset has been created
            And I check that the model has been created
            And I check that the first node has <children> branches
        """
        headers = ["data", "focus_field", "objective", "output_dir",
                   "children"]
        examples = [
            ['data/iris.csv', 'sepal length', '000004', 'scenario28', 2]]
        show_doc(self.test_scenario28, examples)
        for example in examples:
            example = dict(zip(headers, example))
            show_method(self, self.bigml["method"], example)
            test_pred.i_create_all_resources_with_focus_field(
                self, example["data"], example["objective"],
                example["focus_field"], example["output_dir"])
            test_pred.i_check_create_source(self)
            test_pred.i_check_create_dataset(self, suffix=None)
            test_pred.i_check_create_model(self)
            test_pred.i_check_first_node_children(
                self, example["children"], example["objective"])



# -*- coding: utf-8 -*-
#pylint: disable=locally-disabled,unused-argument,no-member
#
# Copyright 2014-2023 BigML
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.



import os
import re


from bigmler.utils import open_mode
from bigmler.tests.common_steps import shell_execute
from bigmler.tests.world import world, res_filename, ok_, eq_


INDENT = ' ' * 4

def python3_contents(filename, prior_contents, alternative=""):
    """Check for a file that has alternative contents for Python3 and return
       its contents

    """
    directory = os.path.dirname(filename)
    basename = os.path.basename(filename)
    basename_name, basename_ext = basename.split(".")
    filename = os.path.join(directory, "%s_py3%s.%s" % ( \
        basename_name, alternative, basename_ext))
    try:
        with open(filename, open_mode("r")) as file_handler:
            return file_handler.read().strip("\n")
    except IOError:
        return prior_contents


def i_create_output(
    step, output=None, language=None, resource_type='source',
    add_fields=False):
    """Step: I create a reify output for the resource in <language> for
    <resource_type> <add_fields> and log results in <output>
    """
    ok_(output is not None and language is not None)
    resource_id = getattr(world, resource_type)['resource']
    command = ("bigmler reify --id " + resource_id + " --language " +
               language + " --store --output " + output)
    if add_fields:
        command += ' --add-fields'
    shell_execute(command, output)


def i_check_output_file(step, output=None, check_file=None):
    """Step: the <output> file is like <check_file>"""
    ok_(check_file is not None and output is not None)
    check_file = res_filename(check_file)
    output_file = os.path.join(world.directory, os.path.basename(output))
    with open(check_file, open_mode("r")) as check_file_handler:
        check_contents = check_file_handler.read().strip("\n")

    with open(output_file, open_mode("r")) as output_file:
        output_file_contents = output_file.read()

    #strip comments at the beginning of the file
    output_file_contents = re.sub(r'#!.*def\smain\(\):\n', '',
                                  output_file_contents,
                                  flags=re.S).strip("\n")
    output_file_contents = output_file_contents.replace( \
        '\nif __name__ == "__main__":\n    main()', '')

    #strip internally added project id information
    p_str = r'\'project\':\s\'project/[a-f0-9]{24}\',?\s?'
    output_file_contents = re.sub(p_str,
                                  '', output_file_contents,
                                  flags=re.S).strip("\n")
    p_str = r'/[a-f0-9]{24}'
    output_file_contents = re.sub(p_str,
                                  '', output_file_contents,
                                  flags=re.S)
    check_contents = re.sub(p_str,
                            '', check_contents,
                            flags=re.S)
    p_str = r';;.*\n'
    output_file_contents = re.sub(p_str,
                                  '', output_file_contents,
                                  flags=re.S)
    check_contents = re.sub(p_str,
                            '', check_contents,
                            flags=re.S)
    p_str = r'created by.*\n'
    output_file_contents = re.sub(p_str,
                                  '', output_file_contents,
                                  flags=re.S)
    check_contents = re.sub(p_str,
                            '', check_contents,
                            flags=re.S)
    p_str = r'    api = .*?\n'
    output_file_contents = re.sub(p_str,
                                  '    api = BigML()\n', output_file_contents,
                                  flags=re.S).strip("\n")
    output_file_contents = re.sub(r'\n\s*', '\n', output_file_contents)
    check_contents = re.sub(r'\n\s*', '\n', check_contents)
    output_file_contents = output_file_contents.strip("\n").strip()
    check_contents = check_contents.strip("\n").strip()
    #strip ids added in jupyter notebook transformation
    # e.g.: ,\n"id": "3185730b"
    p_str = r',\n"id":\s"[a-f0-9]{8}"'
    output_file_contents = re.sub(p_str,
                                  '', output_file_contents,
                                  flags=re.S)
    check_contents = re.sub(p_str,
                            '', check_contents,
                            flags=re.S)
    if check_contents != output_file_contents:
        with open("%s" % check_file, "w") as bck_file:
            bck_file.write(output_file_contents)
        eq_(check_contents, output_file_contents)


def create_source(filename, output=None, args=None):
    """Step: I create a BigML source with data <filename> and params <args>"""
    args.update({"project": world.project_id})
    source = world.api.create_source(res_filename(filename), args)
    world.source = source
    world.directory = os.path.dirname(output)
    world.output = output
    world.api.ok(world.source)
    world.sources.append(source['resource'])


def create_dataset(filename, output=None, args=None):
    """Step: I create a BigML dataset from a source with data <filename>
    and params <args>
    """
    source = world.api.create_source( \
        res_filename(filename), {"project": world.project_id})
    world.source = source
    world.directory = os.path.dirname(output)
    world.output = output
    world.api.ok(world.source)
    world.sources.append(source['resource'])
    world.dataset = world.api.create_dataset(source, args)
    world.api.ok(world.dataset)
    world.datasets.append(world.dataset['resource'])


def create_dataset_from_dataset(filename, output=None, args=None):
    """Step: I create a BigML dataset from dataset with data <filename>
    and params <args>
    """
    source = world.api.create_source( \
        res_filename(filename), {"project": world.project_id})
    world.source = source
    world.directory = os.path.dirname(output)
    world.output = output
    world.api.ok(world.source)
    world.sources.append(source['resource'])
    world.dataset = world.api.create_dataset(source)
    world.api.ok(world.dataset)
    world.datasets.append(world.dataset['resource'])
    world.dataset = world.api.create_dataset(world.dataset, args)
    world.api.ok(world.dataset)
    world.datasets.append(world.dataset['resource'])


def create_dataset_from_datasets(filename, output=None, args=None):
    """Step: I create a BigML dataset from a list of datasets with data
    <filename> and params <args>
    """
    source = world.api.create_source( \
        res_filename(filename), {"project": world.project_id})
    world.source = source
    world.directory = os.path.dirname(output)
    world.output = output
    world.api.ok(world.source)
    world.sources.append(source['resource'])
    world.dataset = world.api.create_dataset(source)
    world.api.ok(world.dataset)
    world.datasets.append(world.dataset['resource'])
    datasets = [world.dataset['resource']]
    world.dataset = world.api.create_dataset(source)
    world.api.ok(world.dataset)
    world.datasets.append(world.dataset['resource'])
    datasets.append(world.dataset)
    world.dataset = world.api.create_dataset(datasets, args)
    world.api.ok(world.dataset)
    world.datasets.append(world.dataset['resource'])


def create_model(filename, output=None, args=None):
    """Step: I create a BigML model from a dataset with data <filename>
    and params <args>
    """
    source = world.api.create_source( \
        res_filename(filename), {"project": world.project_id})
    world.source = source
    world.directory = os.path.dirname(output)
    world.output = output
    world.api.ok(world.source)
    world.sources.append(source['resource'])
    world.dataset = world.api.create_dataset(source)
    world.api.ok(world.dataset)
    world.datasets.append(world.dataset['resource'])
    world.model = world.api.create_model(world.dataset, args)
    world.api.ok(world.model)
    world.models.append(world.model['resource'])


def create_prediction(filename, input_data= None, output=None, args=None):
    """Step: I create a BigML prediction for <filename> from a model with data
    <input_data> and params <args>
    """
    source = world.api.create_source( \
        res_filename(filename), {"project": world.project_id})
    world.source = source
    world.directory = os.path.dirname(output)
    world.output = output
    world.api.ok(world.source)
    world.sources.append(source['resource'])
    world.dataset = world.api.create_dataset(source)
    world.api.ok(world.dataset)
    world.datasets.append(world.dataset['resource'])
    world.model = world.api.create_model(world.dataset)
    world.api.ok(world.model)
    world.models.append(world.model['resource'])
    world.prediction = world.api.create_prediction( \
        world.model, input_data, args)
    world.api.ok(world.prediction)
    world.predictions.append(world.prediction['resource'])


def create_cluster(filename, output=None, args=None):
    """Step: I create a BigML cluster from a dataset with data <filename>
    and params <args>
    """
    source = world.api.create_source( \
        res_filename(filename), {"project": world.project_id})
    world.source = source
    world.directory = os.path.dirname(output)
    world.output = output
    world.api.ok(world.source)
    world.sources.append(source['resource'])
    world.dataset = world.api.create_dataset(source)
    world.api.ok(world.dataset)
    world.datasets.append(world.dataset['resource'])
    args.update({"cluster_seed": "bigml"})
    world.cluster = world.api.create_cluster(world.dataset, args)
    world.api.ok(world.cluster)
    world.clusters.append(world.cluster['resource'])


def create_anomaly(filename, output=None, args=None):
    """Step: I create a BigML anomaly from a dataset with data <filename> and
    params <args>
    """
    source = world.api.create_source( \
        res_filename(filename), {"project": world.project_id})
    world.source = source
    world.directory = os.path.dirname(output)
    world.output = output
    world.api.ok(world.source)
    world.sources.append(source['resource'])
    world.dataset = world.api.create_dataset(source)
    world.api.ok(world.dataset)
    world.datasets.append(world.dataset['resource'])
    args.update({"seed": "bigml", "anomaly_seed": "bigml"})
    world.anomaly = world.api.create_anomaly(world.dataset, args)
    world.api.ok(world.anomaly)
    world.anomalies.append(world.anomaly['resource'])


def create_centroid(filename, input_data=None, output=None, args=None):
    """Step: I create a BigML centroid for <filename> from a cluster with
    data <input_data> and params <args>
    """
    source = world.api.create_source( \
        res_filename(filename), {"project": world.project_id})
    world.source = source
    world.directory = os.path.dirname(output)
    world.output = output
    world.api.ok(world.source)
    world.sources.append(source['resource'])
    world.dataset = world.api.create_dataset(source)
    world.api.ok(world.dataset)
    world.datasets.append(world.dataset['resource'])
    world.cluster = world.api.create_cluster( \
        world.dataset, {"cluster_seed": "bigml"})
    world.api.ok(world.cluster)
    world.clusters.append(world.cluster['resource'])
    world.centroid = world.api.create_centroid( \
        world.cluster, input_data, args)
    world.api.ok(world.centroid)
    world.centroids.append(world.centroid['resource'])


def create_anomaly_score(filename, input_data=None, output=None, args=None):
    """Step: I create a BigML anomaly score for <filename> from an anomaly
    detector with data <input_data> and params <args>
    """
    source = world.api.create_source( \
        res_filename(filename), {"project": world.project_id})
    world.source = source
    world.directory = os.path.dirname(output)
    world.output = output
    world.api.ok(world.source)
    world.sources.append(source['resource'])
    world.dataset = world.api.create_dataset(source)
    world.api.ok(world.dataset)
    world.datasets.append(world.dataset['resource'])
    world.anomaly = world.api.create_anomaly( \
        world.dataset, {"seed": "bigml", "anomaly_seed": "bigml"})
    world.api.ok(world.anomaly)
    world.anomalies.append(world.anomaly['resource'])
    world.anomaly_score = world.api.create_anomaly_score( \
        world.anomaly, input_data, args)
    world.api.ok(world.anomaly_score)
    world.anomaly_scores.append(world.anomaly_score['resource'])


def create_batch_prediction(filename, output=None, args=None):
    """Step: I create a BigML batch prediction from a model with data
    <filename> and params <args>
    """
    source = world.api.create_source(res_filename(filename))
    world.source = source
    world.directory = os.path.dirname(output)
    world.output = output
    world.api.ok(world.source)
    world.sources.append(source['resource'])
    world.dataset = world.api.create_dataset(source)
    world.api.ok(world.dataset)
    world.datasets.append(world.dataset['resource'])
    world.model = world.api.create_model(world.dataset)
    world.api.ok(world.model)
    world.models.append(world.model['resource'])
    world.batch_prediction = world.api.create_batch_prediction( \
        world.model, world.dataset, args)
    world.api.ok(world.batch_prediction)
    world.batch_predictions.append(world.batch_prediction['resource'])


def create_batch_centroid(filename, output=None, args=None):
    """Step: I create a BigML batch centroid from a cluster with data
    <filename> and params <args>
    """
    source = world.api.create_source( \
        res_filename(filename), {"project": world.project_id})
    world.source = source
    world.directory = os.path.dirname(output)
    world.output = output
    world.api.ok(world.source)
    world.sources.append(source['resource'])
    world.dataset = world.api.create_dataset(source)
    world.api.ok(world.dataset)
    world.datasets.append(world.dataset['resource'])
    world.cluster = world.api.create_cluster( \
        world.dataset, {"cluster_seed": "bigml"})
    world.api.ok(world.cluster)
    world.clusters.append(world.cluster['resource'])
    world.batch_centroid = world.api.create_batch_centroid( \
        world.cluster, world.dataset, args)
    world.api.ok(world.batch_centroid)
    world.batch_centroids.append(world.batch_centroid['resource'])


def create_batch_anomaly_score(filename, output=None, args=None):
    """Step: I create a BigML batch anomaly score from an anomaly detector
    with data <filename> and params <args>
    """
    source = world.api.create_source( \
        res_filename(filename), {"project": world.project_id})
    world.source = source
    world.directory = os.path.dirname(output)
    world.output = output
    world.api.ok(world.source)
    world.sources.append(source['resource'])
    world.dataset = world.api.create_dataset(source)
    world.api.ok(world.dataset)
    world.datasets.append(world.dataset['resource'])
    world.anomaly = world.api.create_anomaly( \
        world.dataset, {"seed": "bigml", "anomaly_seed": "bigml"})
    world.api.ok(world.anomaly)
    world.anomalies.append(world.anomaly['resource'])
    world.batch_anomaly_score = world.api.create_batch_anomaly_score( \
        world.anomaly, world.dataset, args)
    world.api.ok(world.batch_anomaly_score)
    world.batch_anomaly_scores.append(world.batch_anomaly_score['resource'])


def create_evaluation(filename, output=None, args=None):
    """Step: I create a BigML evaluation with data <filename> and params
    <args>
    """
    source = world.api.create_source( \
        res_filename(filename), {"project": world.project_id})
    world.source = source
    world.directory = os.path.dirname(output)
    world.output = output
    world.api.ok(world.source)
    world.sources.append(source['resource'])
    world.dataset = world.api.create_dataset(source)
    world.api.ok(world.dataset)
    world.datasets.append(world.dataset['resource'])
    world.model = world.api.create_model(world.dataset)
    world.api.ok(world.model)
    world.models.append(world.model['resource'])
    world.evaluation = world.api.create_evaluation( \
        world.model, world.dataset, args)
    world.api.ok(world.evaluation)
    world.evaluations.append(world.evaluation['resource'])


def create_ensemble(filename, output=None, args=None):
    """Step: I create a BigML ensemble from a dataset with data <filename>
    and params <args>
    """
    source = world.api.create_source( \
        res_filename(filename), {"project": world.project_id})
    world.source = source
    world.directory = os.path.dirname(output)
    world.output = output
    world.api.ok(world.source)
    world.sources.append(source['resource'])
    world.dataset = world.api.create_dataset(source)
    world.api.ok(world.dataset)
    world.datasets.append(world.dataset['resource'])
    args.update({"seed": "BigML", "ensemble_sample": {"seed": "bigml"}})
    world.ensemble = world.api.create_ensemble(world.dataset, args)
    world.api.ok(world.ensemble)
    world.ensembles.append(world.ensemble['resource'])


def create_evaluation_split(filename, output=None, args=None):
    """Step: I create a BigML evaluation with data <filename> split
    training/test and params <args>
    """
    source = world.api.create_source( \
        res_filename(filename), {"project": world.project_id})
    world.source = source
    world.directory = os.path.dirname(output)
    world.output = output
    world.api.ok(world.source)
    world.sources.append(source['resource'])
    world.dataset = world.api.create_dataset(source)
    world.api.ok(world.dataset)
    world.datasets.append(world.dataset['resource'])
    world.dataset_train = world.api.create_dataset(world.dataset, { \
        'sample_rate': 0.7, 'seed': 'BigML'})
    world.api.ok(world.dataset_train)
    world.datasets.append(world.dataset_train['resource'])
    world.dataset_test = world.api.create_dataset(world.dataset, { \
        'sample_rate': 0.7, 'seed': 'BigML', 'out_of_bag': True})
    world.api.ok(world.dataset_test)
    world.datasets.append(world.dataset_test['resource'])
    world.model = world.api.create_model(world.dataset_train)
    world.api.ok(world.model)
    world.models.append(world.model['resource'])
    world.evaluation = world.api.create_evaluation( \
        world.model, world.dataset_test, args)
    world.api.ok(world.evaluation)
    world.evaluations.append(world.evaluation['resource'])


def create_dataset_from_batch_prediction(filename, output=None, args=None):
    """Step: I create a BigML dataset from a batch prediction from a model
    with data <filename> and params <args>
    """
    source = world.api.create_source(res_filename(filename))
    world.source = source
    world.directory = os.path.dirname(output)
    world.output = output
    world.api.ok(world.source)
    world.sources.append(source['resource'])
    world.dataset = world.api.create_dataset(source)
    world.api.ok(world.dataset)
    world.datasets.append(world.dataset['resource'])
    world.model = world.api.create_model(world.dataset)
    world.api.ok(world.model)
    world.models.append(world.model['resource'])
    world.batch_prediction = world.api.create_batch_prediction( \
        world.model, world.dataset, {"output_dataset": True})
    world.api.ok(world.batch_prediction)
    world.batch_predictions.append(world.batch_prediction['resource'])
    world.batch_prediction_dataset = world.api.get_dataset(
        world.batch_prediction['object']['output_dataset_resource'])
    world.api.ok(world.batch_prediction_dataset)
    world.batch_prediction_dataset = world.api.update_dataset( \
        world.batch_prediction_dataset, args)
    world.api.ok(world.batch_prediction_dataset)
    world.datasets.append(world.batch_prediction_dataset['resource'])


def create_dataset_from_batch_centroid(filename, output=None, args=None):
    """Step: I create a BigML dataset from a batch centroid from a cluster
    with data <filename> and params <args>
    """
    source = world.api.create_source(res_filename(filename))
    world.source = source
    world.directory = os.path.dirname(output)
    world.output = output
    world.api.ok(world.source)
    world.sources.append(source['resource'])
    world.dataset = world.api.create_dataset(source)
    world.api.ok(world.dataset)
    world.datasets.append(world.dataset['resource'])
    world.cluster = world.api.create_cluster \
        (world.dataset, {"cluster_seed": "bigml"})
    world.api.ok(world.cluster)
    world.clusters.append(world.cluster['resource'])
    world.batch_centroid = world.api.create_batch_centroid( \
        world.cluster, world.dataset, {"output_dataset": True})
    world.api.ok(world.batch_centroid)
    world.batch_centroids.append(world.batch_centroid['resource'])
    world.batch_centroid_dataset = world.api.get_dataset(
        world.batch_centroid['object']['output_dataset_resource'])
    world.api.ok(world.batch_centroid_dataset)
    world.batch_centroid_dataset = world.api.update_dataset( \
        world.batch_centroid_dataset, args)
    world.api.ok(world.batch_centroid_dataset)
    world.datasets.append(world.batch_centroid_dataset['resource'])


def create_dataset_from_batch_anomaly(filename, output=None, args=None):
    """Step: I create a BigML dataset from a batch anomaly score from an
    anomaly detector with data <filename> and params <args>
    """
    source = world.api.create_source(res_filename(filename))
    world.source = source
    world.directory = os.path.dirname(output)
    world.output = output
    world.api.ok(world.source)
    world.sources.append(source['resource'])
    world.dataset = world.api.create_dataset(source)
    world.api.ok(world.dataset)
    world.datasets.append(world.dataset['resource'])
    world.anomaly = world.api.create_anomaly(world.dataset,
                                             {"seed": "bigml",
                                              "anomaly_seed": "bigml"})
    world.api.ok(world.anomaly)
    world.anomalies.append(world.anomaly['resource'])
    world.batch_anomaly_score = world.api.create_batch_anomaly_score( \
        world.anomaly, world.dataset, {"output_dataset": True})
    world.api.ok(world.batch_anomaly_score)
    world.batch_anomaly_scores.append(world.batch_anomaly_score['resource'])
    world.batch_anomaly_score_dataset = world.api.get_dataset(
        world.batch_anomaly_score['object']['output_dataset_resource'])
    world.api.ok(world.batch_anomaly_score_dataset)
    world.batch_anomaly_score_dataset = world.api.update_dataset( \
        world.batch_anomaly_score_dataset, args)
    world.api.ok(world.batch_anomaly_score_dataset)
    world.datasets.append(world.batch_anomaly_score_dataset['resource'])


def create_dataset_from_dataset_from_batch_centroid(
    filename, output=None, args=None):
    """Step: I create a BigML dataset from a dataset from a batch centroid
    from a cluster with data <filename> and params <args>
    """
    source = world.api.create_source(res_filename(filename))
    world.source = source
    world.directory = os.path.dirname(output)
    world.output = output
    world.api.ok(world.source)
    world.sources.append(source['resource'])
    world.dataset = world.api.create_dataset(source)
    world.api.ok(world.dataset)
    world.datasets.append(world.dataset['resource'])
    world.cluster = world.api.create_cluster( \
        world.dataset, {"cluster_seed": "bigml"})
    world.api.ok(world.cluster)
    world.clusters.append(world.cluster['resource'])
    world.batch_centroid = world.api.create_batch_centroid( \
        world.cluster, world.dataset, {"output_dataset": True})
    world.api.ok(world.batch_centroid)
    world.batch_centroids.append(world.batch_centroid['resource'])
    world.batch_centroid_dataset = world.api.get_dataset(
        world.batch_centroid['object']['output_dataset_resource'])
    world.api.ok(world.batch_centroid_dataset)
    world.datasets.append(world.batch_centroid_dataset['resource'])
    world.dataset = world.api.create_dataset( \
        world.batch_centroid_dataset['resource'], args)
    world.api.ok(world.dataset)
    world.datasets.append(world.dataset['resource'])



# -*- coding: utf-8 -*-
#
# Copyright 2020-2023 BigML
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
"""Resources management functions

"""


import sys

import bigml.api

from bigmler.utils import (dated, get_url, log_message, check_resource,
                           check_resource_error, log_created_resources)
from bigmler.reports import report

from bigmler.resourcesapi.common import set_basic_args, update_json_args


def set_forecast_args(args, fields=None):
    """Return forecast dict

    """
    forecast_args = set_basic_args(args, args.name)
    forecast_args.update({
        "intervals": args.intervals,
    })

    if 'forecast' in args.json_args:
        update_json_args(
            forecast_args,
            args.json_args.get('forecast'),
            fields)

    return forecast_args


def create_forecast(time_series, input_data, forecast_args, args,
                    api=None, session_file=None,
                    path=None, log=None):
    """Creates remote forecast

    """
    if api is None:
        api = bigml.api.BigML()

    message = dated("Creating remote forecast.\n")
    log_message(message, log_file=session_file, console=args.verbosity)
    forecast = api.create_forecast(time_series, input_data,
                                   forecast_args,
                                   retries=None)
    log_created_resources("forecast", path,
                          bigml.api.get_forecast_id(forecast),
                          mode='a')
    forecast_id = check_resource_error(
        forecast, "Failed to create forecast: ")
    try:
        forecast = check_resource(forecast, api.get_forecast,
                                  raise_on_error=True)
    except Exception as exception:
        sys.exit("Failed to get a finished forecast: %s"
                 % str(exception))
    message = dated("Forecast created: %s\n"
                    % get_url(forecast))
    log_message(message, log_file=session_file, console=args.verbosity)
    log_message("%s\n" % forecast_id, log_file=log)
    if args.reports:
        report(args.reports, path, forecast)
    return forecast



# -*- coding: utf-8 -*-
#!/usr/bin/env python
#pylint: disable=locally-disabled,line-too-long,attribute-defined-outside-init
#
# Copyright 2015-2023 BigML
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.


""" Testing Multi-label ensembles

"""
from bigmler.tests.world import (world, common_setup_module,
                                 common_teardown_module, show_method)

import bigmler.tests.basic_tst_prediction_steps as test_pred
import bigmler.tests.ml_tst_prediction_steps as ml_pred


def setup_module():
    """Setup for the module

    """
    common_setup_module()
    test = TestMLEnsembles()
    test.bigml = {"method": "setup_scenario1"}
    test.setup_scenario1()

def teardown_module():
    """Teardown for the module

    """
    common_teardown_module()


class TestMLEnsembles:
    """Testing Multi label Ensembles"""

    def setup_method(self, method):
        """
            Debug information
        """
        self.bigml = {}
        self.bigml["method"] = method.__name__
        print("\n-------------------\nTests in: %s\n" % __name__)

    def teardown_method(self):
        """Calling generic teardown for every method

        """
        world.clear_paths()
        print("\nEnd of tests in: %s\n-------------------\n" % __name__)
        self.bigml = {}

    def setup_scenario1(self):
        """
        Scenario: Successfully building multi-label test predictions from start:
            Given I create BigML multi-label resources tagged as "<tag>" with "<label_separator>" label separator and <number_of_labels> labels uploading train "<data>" file with "<training_separator>" field separator and <number_of_models> models ensembles to test "<test>" and log predictions in "<output>"
            And I check that the source has been created
            And I check that the dataset has been created
            And I check that the models in the ensembles have been created
            Then I check that the predictions are ready
        """
        print(self.setup_scenario1.__doc__)
        headers = ["tag", "label_separator", "number_of_labels", "data",
                   "training_separator", "number_of_models", "test", "output"]
        examples = [
            ['my_multilabel_1', ':', '7', 'data/multilabel.csv', ',', '10',
             'data/test_multilabel.csv', 'scenario_mle_1/predictions.csv']]
        for example in examples:
            example = dict(zip(headers, example))
            show_method(self, self.bigml["method"], example)
            ml_pred.i_create_all_ml_resources_and_ensembles(
                self, tag=example["tag"],
                label_separator=example["label_separator"],
                number_of_labels=example["number_of_labels"],
                data=example["data"],
                training_separator=example["training_separator"],
                number_of_models=example["number_of_models"],
                test=example["test"], output=example["output"])
            test_pred.i_check_create_source(self)
            test_pred.i_check_create_dataset(self)
            test_pred.i_check_create_models_in_ensembles(
                self, in_ensemble=True)
            test_pred.i_check_create_predictions(self)

    def test_scenario2(self):
        """
        Scenario: Successfully building test predictions from source
            Given I have previously executed "<scenario>" or reproduce it with arguments <kwargs>
            And I create BigML multi-label resources using source and <number_of_models> models ensembles to test "<test>" and log predictions in "<output>"
            And I check that the dataset has been created
            And I check that the models in the ensembles have been created
            Then I check that the predictions are ready
        """
        print(self.test_scenario2.__doc__)
        headers = ["scenario", "kwargs", "number_of_models", "test", "output"]
        examples = [
            ['scenario_mle_1', '{"tag": "my_multilabel_1",' +
             ' "data": "data/multilabel.csv", "label_separator": ":",' +
             ' "number_of_labels": 7, "training_separator": ",",' +
             ' "output": "scenario_mle_1/predictions.csv",' +
             ' "test": "data/test_multilabel.csv", "number_of_models": 10}',
             '10', 'data/test_multilabel.csv',
             'scenario_mle_2/predictions.csv']]
        for example in examples:
            example = dict(zip(headers, example))
            show_method(self, self.bigml["method"], example)
            test_pred.i_have_previous_scenario_or_reproduce_it(
                self, example["scenario"], example["kwargs"])
            ml_pred.i_create_resources_and_ensembles_from_source(
                self, multi_label='multi-label ',
                number_of_models=example["number_of_models"],
                test=example["test"], output=example["output"])
            test_pred.i_check_create_dataset(self)
            test_pred.i_check_create_models_in_ensembles(
                self, in_ensemble=True)
            test_pred.i_check_create_predictions(self)

    def test_scenario3(self):
        """
        Scenario: Successfully building test predictions from dataset
            Given I have previously executed "<scenario>" or reproduce it with arguments <kwargs>
            And I create BigML multi-label resources using dataset and <number_of_models> models ensembles to test "<test>" and log predictions in "<output>"
            And I check that the models in the ensembles have been created
            Then I check that the predictions are ready
        """
        print(self.test_scenario3.__doc__)
        headers = ["scenario", "kwargs", "number_of_models", "test", "output"]
        examples = [
            ['scenario_mle_1', '{"tag": "my_multilabel_1",' +
             ' "data": "data/multilabel.csv", "label_separator": ":",' +
             ' "number_of_labels": 7, "training_separator": ",",' +
             ' "output": "scenario_mle_1/predictions.csv",' +
             ' "test": "data/test_multilabel.csv", "number_of_models": 10}',
             '10', 'data/test_multilabel.csv',
             'scenario_mle_3/predictions.csv']]
        for example in examples:
            example = dict(zip(headers, example))
            show_method(self, self.bigml["method"], example)
            test_pred.i_have_previous_scenario_or_reproduce_it(
                self, example["scenario"], example["kwargs"])
            ml_pred.i_create_resources_and_ensembles_from_source(
                self, multi_label='multi-label',
                number_of_models=example["number_of_models"],
                test=example["test"], output=example["output"])
            test_pred.i_check_create_dataset(self)
            test_pred.i_check_create_models_in_ensembles(
                self, in_ensemble=True)
            test_pred.i_check_create_predictions(self)



# -*- coding: utf-8 -*-
#
# Copyright 2020-2023 BigML
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

"""BigMLer - Resource processing: creation, update and retrieval of external
connectors

"""


import bigml.api
import bigmler.resourcesapi.external_connectors as r
import bigmler.utils as u
import bigmler.checkpoint as c


def connector_processing(api, args, resume,
                         session_file=None, path=None, log=None):
    """Creating or retrieving an external connector from input arguments

    """
    # if no external connection info given by the user, we skip
    # processing and no connector will be created
    if not u.has_connection_info(args) and args.external_connector_id is None:
        return None
    external_connector_id = None
    if u.has_connection_info(args):
        # If resuming, try to extract args.external_connector_id form log files

        if resume:
            message = u.dated("External connector ID not found. Resuming.\n")
            resume, external_connector_id = c.checkpoint(
                c.is_external_connector_created, path,
                debug=args.debug, message=message,
                log_file=session_file, console=args.verbosity)

    else:
        external_connector_id = bigml.api.get_external_connector_id( \
            args.external_connector_id)

    # If no external connector is found, we create a new one.
    if external_connector_id is None:
        connector_args = r.set_external_connector_args(\
            args, name=args.name)
        connector = r.create_external_connector(
            connector_args, args, api, session_file, path, log)
        external_connector_id = connector['resource']

    return external_connector_id


def update_external_connector(args, api, resume,
                              session_file=None, path=None, log=None):
    """Updating external connector attributes according to input arguments

    """
    # if no external connector info given by the user, we skip processing and
    # no update will be performed
    if args.external_connector_id is None:
        return None
        # If resuming, try to extract args.external_connector_id form log files

    if resume:
        message = u.dated("External connector not found. Resuming.\n")
        resume, external_connector_id = c.checkpoint(
            c.is_external_connector_created, path,
            debug=args.debug, message=message,
            log_file=session_file, console=args.verbosity)
    elif args.external_connector_id:
        external_connector_id = bigml.api.get_external_connector_id( \
            args.external_connector_id)

    if external_connector_id is not None:
        external_connector_args = r.set_basic_args(args, args.name)
        external_connector = r.update_external_connector(
            external_connector_args, args, api, session_file, log)
        external_connector_id = external_connector['resource']

    return external_connector_id



# -*- coding: utf-8 -*-
#!/usr/bin/env python
#pylint: disable=locally-disabled,line-too-long,attribute-defined-outside-init
#
# Copyright 2015-2023 BigML
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.


""" Testing samples

"""

from bigmler.tests.world import (world, common_setup_module,
                                 common_teardown_module, show_method)

import bigmler.tests.basic_tst_prediction_steps as test_pred
import bigmler.tests.sample_steps as test_sample
import bigmler.tests.dataset_advanced_steps as dataset


def setup_module():
    """Setup for the module

    """
    common_setup_module()

def teardown_module():
    """Teardown for the module

    """
    common_teardown_module()

class TestSamples:
    """Testing sample commands"""

    def setup_method(self, method):
        """
            Debug information
        """
        self.bigml = {}
        self.bigml["method"] = method.__name__
        print("\n-------------------\nTests in: %s\n" % __name__)

    def teardown_method(self):
        """Calling generic teardown for every method

        """
        world.clear_paths()
        print("\nEnd of tests in: %s\n-------------------\n" % __name__)
        self.bigml = {}

    def test_scenario1(self):
        """
        Scenario: Successfully building a new sample from a dataset
            Given I create a BigML dataset from "<data>" and store logs in "<output_dir>"
            And I check that the source has been created
            And I check that the dataset has been created
            Then I create a new sample from the dataset and get the sample using options "<options>" storing logs in "<output_dir>"
            And I check that the sample has been created
            And the sample file is like "<sample_CSV>"
        """
        print(self.test_scenario1.__doc__)
        headers = ["data", "output_dir", "options", "sample_CSV"]
        examples = [
            ['data/iris.csv', 'scenario_smp_1',
             '--occurrence --sample-header --row-index',
             'check_files/sample_iris.csv'],
            ['data/iris.csv', 'scenario_smp_2',
             '--precision 0 --rows 10 --row-offset 10 --unique',
             'check_files/sample_iris2.csv'],
            ['data/iris.csv', 'scenario_smp_3',
             '--row-order-by="-petal length" --row-fields ' +
             '"petal length,petal width" --mode linear',
             'check_files/sample_iris3.csv']]
        for example in examples:
            example = dict(zip(headers, example))
            show_method(self, self.bigml["method"], example)
            dataset.i_create_dataset(self, data=example["data"],
                                     output_dir=example["output_dir"])
            test_pred.i_check_create_source(self)
            test_pred.i_check_create_dataset(self)
            test_sample.i_create_sample(
                self, options=example["options"],
                output_dir=example["output_dir"])
            test_sample.i_check_create_sample(self)
            test_sample.i_check_sample_file(
                self, check_sample_file=example["sample_CSV"])

    def test_scenario2(self):
        """
        Scenario: Successfully building a new sample from a dataset
            Given I create a BigML dataset from "<data>" and store logs in "<output_dir>"
            And I check that the source has been created
            And I check that the dataset has been created
            Then I create a new sample from the dataset and get the sample using options "<options>" storing logs in "<output_dir>"
            And I check that the sample has been created
            And the sample JSON is like the one in "<sample_JSON_file>"
        """
        print(self.test_scenario2.__doc__)
        headers = ["data", "output_dir", "options", "sample_JSON"]
        examples = [
            ['data/iris.csv', 'scenario_smp_4',
             '--stat-field "petal length"',
             'check_files/stat_info.json']]
        for example in examples:
            example = dict(zip(headers, example))
            show_method(self, self.bigml["method"], example)
            dataset.i_create_dataset(
                self, data=example["data"], output_dir=example["output_dir"])
            test_pred.i_check_create_source(self)
            test_pred.i_check_create_dataset(self)
            test_sample.i_create_sample(
                self, options=example["options"],
                output_dir=example["output_dir"])
            test_sample.i_check_create_sample(self)
            test_sample.i_check_sample_json(
                self, check_sample_file=example["sample_JSON"])



# -*- coding: utf-8 -*-
#!/usr/bin/env python
#pylint: disable=locally-disabled,line-too-long,attribute-defined-outside-init
#pylint: disable=locally-disabled,invalid-name
#
# Copyright 2020-2023 BigML
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.


""" Testing fusion predictions creation

"""


import os

from bigmler.tests.world import (world, common_setup_module,
                                 common_teardown_module, show_method)


import bigmler.tests.external_connector_steps as external_connection
import bigmler.tests.basic_tst_prediction_steps as prediction_create

HOST = os.getenv("BIGML_EXTERNAL_CONN_HOST")
PORT = os.getenv("BIGML_EXTERNAL_CONN_PORT")
DATABASE = os.getenv("BIGML_EXTERNAL_CONN_DB")
USER = os.getenv("BIGML_EXTERNAL_CONN_USER")
PASSWORD = os.getenv("BIGML_EXTERNAL_CONN_PWD")

if HOST is None or PORT is None or DATABASE is None or USER is None or \
        PASSWORD is None:
    raise ValueError("The external connections tests needs defining some"
                     " environment variables: BIGML_EXTERNAL_CONN_HOST,"
                     " BIGML_EXTERNAL_CONN_PORT, BIGML_EXTERNAL_CONN_DB,"
                     " BIGML_EXTERNAL_CONN_USER, BIGML_EXTERNAL_CONN_PWD")


def setup_module():
    """Setup for the module

    """
    common_setup_module()
    TestExternalConnector()


def teardown_module():
    """Teardown for the module

    """
    common_teardown_module()


class TestExternalConnector:
    """Testing external connectors"""

    def setup_method(self, method):
        """
            Debug information
        """
        self.bigml = {}
        self.bigml["method"] = method.__name__
        print("\n-------------------\nTests in: %s\n" % __name__)

    def teardown_method(self):
        """Calling generic teardown for every method

        """
        world.clear_paths()
        print("\nEnd of tests in: %s\n-------------------\n" % __name__)
        self.bigml = {}

    def test_scenario01(self):
        """
        Scenario: Successfully building an external connector
            And I create BigML external connection using  "<name>", "<source>", "<host>", "<port>", "<user>", "<password>" and "<database>" and log files in  "<output_dir>"
            And I check that the external connection is ready
            And I update the external connection to "<new_name>"
            Then the external connection has name "<new_name>"
            And I create a source from the external connector
            And I check that the source is ready

        """
        print(self.test_scenario01.__doc__)
        headers = ["name", "source", "host", "port", "user", "password",
                   "database", "output_dir", "new_name", "connector",
                   "sql_query"]
        examples = [
            ['my connection', 'postgresql', HOST,
            PORT, USER, PASSWORD, DATABASE, 'scenario_41',
            'my new connection', 'scenario_41/my_connector.json',
            'select * from iris']]
        for example in examples:
            example = dict(zip(headers, example))
            show_method(self, self.bigml["method"], example)
            external_connection.i_create_external_connector( \
                self, name=example["name"], source=example["source"],
                host=example["host"], port=example["port"],
                user=example["user"], password=example["password"],
                database=example["database"], output_dir=example["output_dir"])
            external_connection.i_check_external_connector(self)
            external_connection.i_update_external_connector( \
                self, name=example["new_name"],
                output_dir=example["output_dir"])
            external_connection.i_check_external_connector(self)
            external_connection.i_check_external_connector_name( \
                self, example["new_name"])
            prediction_create.i_create_source_from_connector(
                self, example["connector"], example["output_dir"],
                example["sql_query"])
            prediction_create.i_check_create_source(self)



# -*- coding: utf-8 -*-
#!/usr/bin/env python
#pylint: disable=locally-disabled,line-too-long,attribute-defined-outside-init
#
# Copyright 2015-2023 BigML
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.


""" Testing splitting models according to max categories

"""
from bigmler.tests.world import (world, common_setup_module,
                                 common_teardown_module, show_method)


import bigmler.tests.basic_tst_prediction_steps as test_pred
import bigmler.tests.max_categories_tst_prediction_steps as max_cat


def setup_module():
    """Setup for the module

    """
    common_setup_module()
    test = TestMaxCategories()
    test.bigml = {"method": "setup_scenario1"}
    test.setup_scenario1()


def teardown_module():
    """Teardown for the module

    """
    common_teardown_module()


class TestMaxCategories:
    """Testing max categories option """

    def setup_method(self, method):
        """
            Debug information
        """
        self.bigml = {}
        self.bigml["method"] = method.__name__
        print("\n-------------------\nTests in: %s\n" % __name__)

    def teardown_method(self):
        """Calling generic teardown for every method

        """
        world.clear_paths()
        print("\nEnd of tests in: %s\n-------------------\n" % __name__)
        self.bigml = {}

    def setup_scenario1(self):
        """
        Scenario: Successfully building test predictions from training data using datasets with max categories
            Given I create BigML resources from "<data>" with <max_categories> as categories limit and <objective> as objective field to test "<test>" and log predictions in "<output>"
            And I check that the source has been created
            And I check that the dataset has been created
            And I check that the max_categories datasets have been created
            And I check that the models have been created
            And I check that the predictions are ready
            Then the local prediction file is like "<predictions_file>"
        """
        print(self.setup_scenario1.__doc__)
        headers = ["data", "max_categories", "objective", "test", "output",
                   "predictions_file"]
        examples = [
            ['data/iris.csv', '1', 'species', 'data/test_iris.csv',
             'scenario_mc_1/predictions.csv',
             'check_files/predictions_mc.csv']]
        for example in examples:
            example = dict(zip(headers, example))
            show_method(self, self.bigml["method"], example)
            max_cat.i_create_all_mc_resources(
                self, example["data"],
                max_categories=example["max_categories"],
                objective=example["objective"],
                test=example["test"], output=example["output"])
            test_pred.i_check_create_source(self)
            test_pred.i_check_create_dataset(self, suffix=None)
            max_cat.i_check_create_max_categories_datasets(self)
            test_pred.i_check_create_models(self)
            test_pred.i_check_create_predictions(self)
            test_pred.i_check_predictions(self, example["predictions_file"])

    def test_scenario2(self):
        """
        Scenario: Successfully building test predictions from source using datasets with max categories
            Given I have previously executed "<scenario>" or reproduce it with arguments <kwargs>
            And I create BigML resources with <max_categories> as categories limit and <objective> as objective field using source to test "<test>" and log predictions in "<output>"
            And I check that the dataset has been created
            And I check that the max_categories datasets have been created
            And I check that the models have been created
            And I check that the predictions are ready
            Then the local prediction file is like "<predictions_file>"
        """
        print(self.test_scenario2.__doc__)
        headers = ["scenario", "kwargs", "max_categories", "objective", "test",
                   "output", "predictions_file"]
        examples = [
            ['scenario_mc_1', '{"data": "data/iris.csv",' +
             ' "max_categories": "1", "objective": "species",' +
             ' "output": "scenario_mc_1/predictions.csv",' +
             ' "test": "data/test_iris.csv"}', '1', 'species',
             'data/test_iris.csv', 'scenario_mc_2/predictions.csv',
             'check_files/predictions_mc.csv']]
        for example in examples:
            example = dict(zip(headers, example))
            show_method(self, self.bigml["method"], example)
            test_pred.i_have_previous_scenario_or_reproduce_it(
                self, example["scenario"], example["kwargs"])
            max_cat.i_create_all_mc_resources_from_source(
                self, max_categories=example["max_categories"],
                objective=example["objective"], test=example["test"],
                output=example["output"])
            test_pred.i_check_create_dataset(self, suffix=None)
            max_cat.i_check_create_max_categories_datasets(self)
            test_pred.i_check_create_models(self)
            test_pred.i_check_create_predictions(self)
            test_pred.i_check_predictions(self, example["predictions_file"])

    def test_scenario3(self):
        """
        Scenario: Successfully building test predictions from dataset using datasets with max categories
            Given I have previously executed "<scenario>" or reproduce it with arguments <kwargs>
            And I create BigML resources with <max_categories> as categories limit and <objective> as objective field using dataset to test "<test>" and log predictions in "<output>"
            And I check that the max_categories datasets have been created
            And I check that the models have been created
            And I check that the predictions are ready
            Then the local prediction file is like "<predictions_file>"
        """
        print(self.test_scenario3.__doc__)
        headers = ["scenario", "kwargs", "max_categories", "objective", "test",
                   "output", "predictions_file"]
        examples = [
            ['scenario_mc_1', '{"data": "data/iris.csv",' +
             ' "max_categories": "1", "objective": "species",' +
             ' "output": "scenario_mc_1/predictions.csv",' +
             ' "test": "data/test_iris.csv"}', '1', 'species',
             'data/test_iris.csv', 'scenario_mc_3/predictions.csv',
             'check_files/predictions_mc.csv']]
        for example in examples:
            example = dict(zip(headers, example))
            show_method(self, self.bigml["method"], example)
            test_pred.i_have_previous_scenario_or_reproduce_it(
                self, example["scenario"], example["kwargs"])
            max_cat.i_create_all_mc_resources_from_dataset(
                self, max_categories=example["max_categories"],
                objective=example["objective"], test=example["test"],
                output=example["output"])
            max_cat.i_check_create_max_categories_datasets(self)
            test_pred.i_check_create_models(self)
            test_pred.i_check_create_predictions(self)
            test_pred.i_check_predictions(self, example["predictions_file"])

    def test_scenario4(self):
        """
        Scenario: Successfully building ensembles test predictions from models file with max categories
            Given I have previously executed "<scenario>" or reproduce it with arguments <kwargs>
            And I create BigML resources using models in file "<models_file>" to test "<test>" and log predictions with combine method in "<output>"
            And I check that the predictions are ready
            Then the local prediction file is like "<predictions_file>"
        """
        print(self.test_scenario4.__doc__)
        headers = ["scenario", "kwargs", "models_file", "test", "output",
                   "predictions_file"]
        examples = [
            ['scenario_mc_1', '{"data": "data/iris.csv",' +
             ' "max_categories": "1", "objective": "species",' +
             ' "output": "scenario_mc_1/predictions.csv",' +
             ' "test": "data/test_iris.csv"}', 'scenario_mc_1/models',
             'data/test_iris.csv', 'scenario_mc_4/predictions.csv',
             'check_files/predictions_mc.csv']]
        for example in examples:
            example = dict(zip(headers, example))
            show_method(self, self.bigml["method"], example)
            test_pred.i_have_previous_scenario_or_reproduce_it(
                self, example["scenario"], example["kwargs"])
            max_cat.i_create_all_mc_resources_from_models(
                self, models_file=example["models_file"], test=example["test"],
                output=example["output"])
            test_pred.i_check_create_predictions(self)
            test_pred.i_check_predictions(self, example["predictions_file"])

    def test_scenario5(self):
        """
        Scenario: Successfully building test predictions from dataset using datasets and model fields with max categories
            Given I have previously executed "<scenario>" or reproduce it with arguments <kwargs>
            And I create BigML resources with <max_categories> as categories limit and <objective> as objective field and model fields "<model_fields>" using dataset to test "<test>" and log predictions in "<output>"
            And I check that the max_categories datasets have been created
            And I check that the models have been created
            And I check that the predictions are ready
            Then the local prediction file is like "<predictions_file>"
        """
        print(self.test_scenario5.__doc__)
        headers = ["scenario", "kwargs", "max_categories", "objective",
                   "model_fields", "test", "output", "predictions_file"]
        examples = [
            ['scenario_mc_1', '{"data": "data/iris.csv",' +
             ' "max_categories": "1", "objective": "species",' +
             ' "output": "scenario_mc_1/predictions.csv",' +
             ' "test": "data/test_iris.csv"}', '1', 'species',
             'sepal length,sepal width', 'data/test_iris.csv',
             'scenario_mc_5/predictions.csv',
             'check_files/predictions_mc2.csv']]
        for example in examples:
            example = dict(zip(headers, example))
            show_method(self, self.bigml["method"], example)
            test_pred.i_have_previous_scenario_or_reproduce_it(
                self, example["scenario"], example["kwargs"])
            max_cat.i_create_all_mc_resources_from_dataset_with_model_fields(
                self, max_categories=example["max_categories"],
                objective=example["objective"],
                model_fields=example["model_fields"], test=example["test"],
                output=example["output"])
            max_cat.i_check_create_max_categories_datasets(self)
            test_pred.i_check_create_models(self)
            test_pred.i_check_create_predictions(self)
            test_pred.i_check_predictions(self, example["predictions_file"])



# -*- coding: utf-8 -*-
#
# Copyright 2020-2023 BigML
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
"""Resources management functions

"""


import sys

import bigml.api

from bigmler.utils import (dated, get_url, log_message, check_resource,
                           plural, is_shared,
                           check_resource_error, log_created_resources)
from bigmler.reports import report

from bigmler.resourcesapi.common import set_basic_model_args, \
    update_json_args, wait_for_available_tasks

from bigmler.resourcesapi.common import FIELDS_QS, EVALUATE_SAMPLE_RATE, \
    ALL_FIELDS_QS


def set_time_series_args(args, name=None, fields=None,
                         objective_id=None):
    """Return time-series arguments dict

    """
    if name is None:
        name = args.name
    if objective_id is None:
        objective_id = args.objective_id_

    time_series_args = set_basic_model_args(args, name)
    time_series_args.update({
        "all_numeric_objectives": args.all_numeric_objectives,
        "period": args.period
    })

    # if we need to evaluate and there's no previous split, use a range
    if args.evaluate and args.test_split == 0 and not args.has_test_datasets_:
        args.range_ = [1, int(args.max_rows * EVALUATE_SAMPLE_RATE)]
    if objective_id is not None:
        time_series_args.update({"objective_field": objective_id})
    if args.objectives:
        time_series_args.update({"objective_fields": args.objective_fields_})
    if args.damped_trend is not None:
        time_series_args.update({"damped_trend": args.damped_trend})
    if args.error is not None:
        time_series_args.update({"error": args.error})
    if args.field_parameters:
        time_series_args.update({"field_parameters": args.field_parameters_})
    if args.range_:
        time_series_args.update({"range": args.range_})
    if args.seasonality is not None:
        time_series_args.update({"seasonality": args.seasonality})
    if args.trend is not None:
        time_series_args.update({"trend": args.trend})

    if args.time_start or args.time_end or args.time_interval or \
            args.time_interval_unit:
        time_range = {}
        if args.time_start:
            time_range.update({"start": args.time_start})
        if args.time_end:
            time_range.update({"end": args.time_end})
        if args.time_interval:
            time_range.update({"interval": args.time_interval})
        if args.time_interval_unit:
            time_range.update({"interval_unit": args.time_interval_unit})
        time_series_args.update({"time_range": time_range})

    if 'time_series' in args.json_args:
        update_json_args(time_series_args,
                         args.json_args.get('time_series'),
                         fields)
    return time_series_args


def create_time_series(datasets, time_series_ids,
                       time_series_args,
                       args, api=None, path=None,
                       session_file=None, log=None):
    """Create remote time-series

    """
    if api is None:
        api = bigml.api.BigML()

    time_series_set = time_series_ids[:]
    existing_time_series = len(time_series_set)
    time_series_args_list = []
    datasets = datasets[existing_time_series:]
    # if resuming and all time-series were created,
    # there will be no datasets left
    if datasets:
        if isinstance(time_series_args, list):
            time_series_args_list = time_series_args

        # Only one time-series per command, at present
        number_of_time_series = 1
        message = dated("Creating %s time-series.\n" %
                        number_of_time_series)
        log_message(message, log_file=session_file,
                    console=args.verbosity)

        query_string = FIELDS_QS
        inprogress = []
        for i in range(0, number_of_time_series):
            wait_for_available_tasks(inprogress,
                                     args.max_parallel_time_series,
                                     api, "timeseries")
            if time_series_args_list:
                time_series_args = time_series_args_list[i]

            if (args.test_datasets and args.evaluate):
                dataset = datasets[i]
                time_series = api.create_time_series( \
                    dataset, time_series_args, retries=None)
            else:
                time_series = api.create_time_series( \
                datasets, time_series_args, retries=None)
            time_series_id = check_resource_error( \
                time_series, "Failed to create time-series: ")
            log_message("%s\n" % time_series_id, log_file=log)
            time_series_ids.append(time_series_id)
            inprogress.append(time_series_id)
            time_series_set.append(time_series)
            log_created_resources("time_series",
                                  path,
                                  time_series_id, mode='a')

        if args.verbosity:
            if bigml.api.get_status(time_series)['code'] != \
                    bigml.api.FINISHED:
                try:
                    time_series = check_resource( \
                        time_series, api.get_time_series,
                        query_string=query_string,
                        raise_on_error=True)
                except Exception as exception:
                    sys.exit("Failed to get a finished time-series:"
                             " %s" %
                             str(exception))
                time_series_set[0] = time_series
            message = dated("Time-series created: %s\n" %
                            get_url(time_series))
            log_message(message, log_file=session_file,
                        console=args.verbosity)
            if args.reports:
                report(args.reports, path, time_series)

    return time_series_set, time_series_ids


def get_time_series(time_series_ids,
                    args, api=None, session_file=None):
    """Retrieves remote time-series in its actual status

    """
    if api is None:
        api = bigml.api.BigML()

    time_series_id = ""
    time_series_set = time_series_ids
    time_series_id = time_series_ids[0]
    message = dated("Retrieving %s. %s\n" %
                    (plural("time-series", len(time_series_ids)),
                     get_url(time_series_id)))
    log_message(message, log_file=session_file, console=args.verbosity)
    # only one time-series to predict at present
    try:
        # we need the whole fields structure when exporting fields
        query_string = FIELDS_QS if not args.export_fields else ALL_FIELDS_QS
        time_series = check_resource(time_series_ids[0],
                                     api.get_time_series,
                                     query_string=query_string,
                                     raise_on_error=True)
    except Exception as exception:
        sys.exit("Failed to get a finished time-series: %s" % \
            str(exception))
    time_series_set[0] = time_series

    return time_series_set, time_series_ids


def set_publish_time_series_args(args):
    """Set args to publish time-series

    """
    public_time_series = {}
    if args.public_time_series:
        public_time_series = {"private": False}
        if args.model_price:
            public_time_series.update(price=args.model_price)
        if args.cpp:
            public_time_series.update(credits_per_prediction=args.cpp)
    return public_time_series


def update_time_series(time_series, time_series_args,
                       args, api=None, path=None, session_file=None):
    """Updates time-series properties

    """
    if api is None:
        api = bigml.api.BigML()

    message = dated("Updating time-series. %s\n" %
                    get_url(time_series))
    log_message(message, log_file=session_file,
                console=args.verbosity)
    time_series = api.update_time_series(time_series, \
        time_series_args)
    check_resource_error(time_series,
                         "Failed to update time-series: %s"
                         % time_series['resource'])
    time_series = check_resource(time_series,
                                 api.get_time_series,
                                 query_string=FIELDS_QS,
                                 raise_on_error=True)
    if is_shared(time_series):
        message = dated("Shared time-series link. %s\n" %
                        get_url(time_series, shared=True))
        log_message(message, log_file=session_file, console=args.verbosity)
        if args.reports:
            report(args.reports, path, time_series)

    return time_series



# -*- coding: utf-8 -*-
#!/usr/bin/env python
#pylint: disable=locally-disabled,line-too-long,attribute-defined-outside-init
#
# Copyright 2015-2023 BigML
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.


""" Testing Multi-label predictions

"""

from bigmler.tests.world import (world, common_setup_module,
                                 common_teardown_module, show_method)

import bigmler.tests.basic_tst_prediction_steps as test_pred
import bigmler.tests.ml_tst_prediction_steps as ml_pred


def setup_module():
    """Setup for the module

    """
    common_setup_module()
    test = TestML()
    test.bigml = {"method": "setup_scenario1"}
    test.setup_scenario1()


def teardown_module():
    """Teardown for the module

    """
    common_teardown_module()

class TestML:
    """Test multi labels"""

    def setup_method(self, method):
        """
            Debug information
        """
        self.bigml = {}
        self.bigml["method"] = method.__name__
        print("\n-------------------\nTests in: %s\n" % __name__)

    def teardown_method(self):
        """Calling generic teardown for every method

        """
        world.clear_paths()
        print("\nEnd of tests in: %s\n-------------------\n" % __name__)
        self.bigml = {}

    def setup_scenario1(self):
        """
        Scenario: Successfully building multi-label test predictions from start:
            Given I create BigML multi-label resources tagged as "<tag>" with "<label_separator>" label separator and <number_of_labels> labels uploading train "<data>" file with "<training_separator>" field separator to test "<test>" and log predictions in "<output>"
            And I check that the source has been created
            And I check that the dataset has been created
            And I check that the models have been created
            And I check that the predictions are ready
            Then the local prediction file is like "<predictions_file>"
        """
        print(self.setup_scenario1.__doc__)
        headers = ["tag", "label_separator", "number_of_labels", "data",
                   "training_separator", "test", "output", "predictions_file"]
        examples = [
            ['my_multilabel_1', ':', '7', 'data/multilabel.csv', ',',
             'data/test_multilabel.csv', 'scenario_ml_1/predictions.csv',
             'check_files/predictions_ml.csv']]
        for example in examples:
            example = dict(zip(headers, example))
            show_method(self, self.bigml["method"], example)
            kwargs = example.copy()
            del kwargs["predictions_file"]
            ml_pred.i_create_all_ml_resources(
                self, **kwargs)
            test_pred.i_check_create_source(self)
            test_pred.i_check_create_dataset(self)
            test_pred.i_check_create_models(self)
            test_pred.i_check_create_predictions(self)
            test_pred.i_check_predictions(self, example["predictions_file"])

    def test_scenario2(self):
        """
        Scenario: Successfully building test predictions from source
            Given I have previously executed "<scenario>" or reproduce it with arguments <kwargs>
            And I create BigML multi-label resources using source to test "<test>" and log predictions in "<output>"
            And I check that the dataset has been created
            And I check that the models have been created
            And I check that the predictions are ready
            Then the local prediction file is like "<predictions_file>"

            Examples:
            |scenario    | kwargs                                                  | test                    | output                        |predictions_file           |
            | scenario_ml_1| {"tag": "my_multilabel_1", "data": "../data/multilabel.csv", "label_separator": ":", "number_of_labels": 7, "training_separator": ",", "output": "./scenario_ml_1/predictions.csv", "test": "../data/test_multilabel.csv"}   | ../data/test_multilabel.csv   | ./scenario_ml_2/predictions.csv   | ./check_files/predictions_ml_comma.csv   |
        """
        print(self.test_scenario2.__doc__)
        headers = ["scenario", "kwargs", "test", "output", "predictions_file"]
        examples = [
            ['scenario_ml_1', '{"tag": "my_multilabel_1",' +
             ' "data": "data/multilabel.csv", "label_separator": ":",' +
             ' "number_of_labels": 7, "training_separator": ",",' +
             ' "output": "scenario_ml_1/predictions.csv",' +
             ' "test": "data/test_multilabel.csv"}',
             'data/test_multilabel.csv', 'scenario_ml_2/predictions.csv',
             'check_files/predictions_ml_comma.csv']]
        for example in examples:
            example = dict(zip(headers, example))
            show_method(self, self.bigml["method"], example)
            test_pred.i_have_previous_scenario_or_reproduce_it(
                self, example["scenario"], example["kwargs"])
            test_pred.i_create_resources_from_source(
                self, multi_label='multi-label',
                test=example["test"], output=example["output"])
            test_pred.i_check_create_dataset(self)
            test_pred.i_check_create_models(self)
            test_pred.i_check_create_predictions(self)
            test_pred.i_check_predictions(self, example["predictions_file"])

    def test_scenario3(self):
        """
        Scenario: Successfully building test predictions from dataset
            Given I have previously executed "<scenario>" or reproduce it with arguments <kwargs>
            And I create BigML multi-label resources using dataset to test "<test>" and log predictions in "<output>"
            And I check that the models have been created
            And I check that the predictions are ready
            Then the local prediction file is like "<predictions_file>"
        """
        print(self.test_scenario3.__doc__)
        headers = ["scenario", "kwargs", "test", "output", "predictions_file"]
        examples = [
            ['scenario_ml_1', '{"tag": "my_multilabel_1",' +
             ' "data": "data/multilabel.csv", "label_separator": ":",' +
             ' "number_of_labels": 7, "training_separator": ",",' +
             ' "output": "scenario_ml_1/predictions.csv",' +
             ' "test": "data/test_multilabel.csv"}',
             'data/test_multilabel.csv', 'scenario_ml_3/predictions.csv',
             'check_files/predictions_ml_comma.csv']]
        for example in examples:
            example = dict(zip(headers, example))
            show_method(self, self.bigml["method"], example)
            test_pred.i_have_previous_scenario_or_reproduce_it(
                self, example["scenario"], example["kwargs"])
            test_pred.i_create_resources_from_dataset(
                self, multi_label='multi-label',
                test=example["test"], output=example["output"])
            test_pred.i_check_create_models(self)
            test_pred.i_check_create_predictions(self)
            test_pred.i_check_predictions(self, example["predictions_file"])

    def test_scenario4(self):
        """
        Scenario: Successfully building test predictions from models file
            Given I have previously executed "<scenario>" or reproduce it with arguments <kwargs>
            And I create BigML multi-label resources using models in file "<models_file>" to test "<test>" and log predictions in "<output>"
            And I check that the predictions are ready
            Then the local prediction file is like "<predictions_file>"
        """
        print(self.test_scenario4.__doc__)
        headers = ["scenario", "kwargs", "models_file", "test", "output",
                   "predictions_file"]
        examples = [
            ['scenario_ml_1', '{"tag": "my_multilabel_1",' +
             ' "data": "data/multilabel.csv", "label_separator": ":",' +
             ' "number_of_labels": 7, "training_separator": ",",' +
             ' "output": "scenario_ml_1/predictions.csv",' +
             ' "test": "data/test_multilabel.csv"}', 'scenario_ml_1/models',
             'data/test_multilabel.csv', 'scenario_ml_4/predictions.csv',
             'check_files/predictions_ml_comma.csv']]
        for example in examples:
            example = dict(zip(headers, example))
            show_method(self, self.bigml["method"], example)
            test_pred.i_have_previous_scenario_or_reproduce_it(
                self, example["scenario"], example["kwargs"])
            test_pred.i_create_resources_from_models_file(
                self, multi_label='multi-label',
                models_file=example["models_file"],
                test=example["test"], output=example["output"])
            test_pred.i_check_create_predictions(self)
            test_pred.i_check_predictions(self, example["predictions_file"])

    def test_scenario5(self):
        """
        Scenario: Successfully building test predictions from models retrieved by tag
            Given I have previously executed "<scenario>" or reproduce it with arguments <kwargs>
            And I create BigML multi-label resources using models tagged as "<tag>" to test "<test>" and log predictions in "<output>"
            And I check that the predictions are ready
            Then the local prediction file is like "<predictions_file>"
        """
        print(self.test_scenario5.__doc__)
        headers = ["scenario", "kwargs", "tag", "test", "output",
                   "predictions_file"]
        examples = [
            ['scenario_ml_6', '{"tag": "my_multilabel_5",' +
             ' "data": "data/multilabel.csv", "label_separator": ":",' +
             ' "number_of_labels": 7, "training_separator": ",",' +
             ' "output": "scenario_ml_6/predictions.csv",' +
             ' "test": "data/test_multilabel.csv"}', 'my_multilabel_5',
             'data/test_multilabel.csv', 'scenario_ml_5/predictions.csv',
             'check_files/predictions_ml_comma.csv']]
        for example in examples:
            example = dict(zip(headers, example))
            show_method(self, self.bigml["method"], example)
            test_pred.i_have_previous_scenario_or_reproduce_it(
                self, example["scenario"], example["kwargs"])
            ml_pred.i_predict_ml_from_model_tag(
                self, tag=example["tag"], test=example["test"],
                output=example["output"])
            test_pred.i_check_create_predictions(self)
            test_pred.i_check_predictions(self, example["predictions_file"])

    def test_scenario6(self):
        """
        Scenario: Successfully building test predictions from models retrieved by tag
            Given I have previously executed "<scenario>" or reproduce it with arguments <kwargs>
            And I create BigML multi-label resources with labels "<labels>" using models tagged as "<tag>" to test "<test>" and log predictions in "<output>"
            And I check that the predictions are ready
            Then the local prediction file is like "<predictions_file>"
        """
        print(self.test_scenario6.__doc__)
        headers = ["scenario", "kwargs", "labels", "tag", "test", "output",
                   "predictions_file"]
        examples = [
            ['scenario_ml_6', '{"tag": "my_multilabel_5",' +
             ' "data": "data/multilabel.csv", "label_separator": ":",' +
             ' "number_of_labels": 7, "training_separator": ",",' +
             ' "output": "scenario_ml_6/predictions.csv",' +
             ' "test": "data/test_multilabel.csv"}', 'Adult,Student',
             'my_multilabel_5', 'data/test_multilabel.csv',
             'scenario_ml_7/predictions.csv',
             'check_files/predictions_ml_labels.csv']]
        for example in examples:
            example = dict(zip(headers, example))
            show_method(self, self.bigml["method"], example)
            test_pred.i_have_previous_scenario_or_reproduce_it(
                self, example["scenario"], example["kwargs"])
            ml_pred.i_predict_ml_from_model_tag_with_labels(
                self, labels=example["labels"], tag=example["tag"],
                test=example["test"], output=example["output"])
            test_pred.i_check_create_predictions(self)
            test_pred.i_check_predictions(self, example["predictions_file"])



# -*- coding: utf-8 -*-
#pylint: disable=locally-disabled,unused-argument,no-member
#
# Copyright 2020-2023 BigML
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.


import os

from bigml.api import check_resource

from bigmler.tests.common_steps import shell_execute
from bigmler.tests.world import world, ok_, eq_


def i_create_external_connector(step, name=None, source=None, host=None,
                                port=None, database=None, user=None,
                                password=None ,output_dir=None):
    """Step: I create BigML external connection using <name>, <source>, <host>,
    <port>, database, user and password and log files in <output_dir>
    """
    ok_(name is not None and source is not None and host is not None and
        port is not None and database is not None and user is not None and
        password is not None and output_dir is not None)
    command = ("bigmler connector --name \"" + name +
               "\" --host " + host +
               " --port " + port +
               " --database " +  database +
               " --user " + user +
               " --password " + password +
               " --engine " + source +
               " --store --output-dir " + output_dir)
    shell_execute(command, "%s/x" % output_dir)


def i_check_external_connector(step):
    """Step: I check that the external connection is ready"""
    connector_file = os.path.join(world.directory, "external_connector")
    try:
        with open(connector_file) as handler:
            connector = check_resource(handler.readline().strip(),
                                       world.api.get_external_connector)
            world.external_connectors.append(connector['resource'])
            world.external_connector = connector
    except Exception as exc:
        ok_(False, msg=str(exc))


def i_update_external_connector(step, name=None, output_dir=None):
    """Step: I update the external connection to <name> and logs to
    <output_dir>
    """
    ok_(name is not None and output_dir is not None)
    command = ("bigmler connector --external-connector " +
               world.external_connector["resource"] +
               " --name \"" + name + "\"" + " --output-dir " + output_dir)
    shell_execute(command, "%s/x" % output_dir)


def i_check_external_connector_name(step, name=None):
    """Step: I check that the external connection is ready"""
    ec_name = world.external_connector.get( \
        "object", world.external_connector).get("name")
    eq_(ec_name, name)



# -*- coding: utf-8 -*-
#pylint: disable=locally-disabled,unused-argument,no-member
#
# Copyright 2021-2023 BigML
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.


import os
import zipfile
import glob


from bigml.api import check_resource
from bigml.util import filter_by_extension
from bigml.fields import Fields
from bigml.constants import IMAGE_EXTENSIONS

from bigmler.tests.common_steps import shell_execute
from bigmler.utils import read_sources
from bigmler.processing.annotations import fields_from_annotations, \
    labels_from_annotations
from bigmler.tests.world import world, res_filename, ok_, eq_


TRANSLATED_FEATURES = {"histogram_of_gradients": "HOG"}


def i_create_composite_from_sources(step, sources=None, output_dir=None):
    """Step: I create BigML composite from a list of sources and
    log results in <output_dir>
    """
    ok_(sources is not None and output_dir is not None)
    command = ("bigmler source --sources " + sources +
               " --store --output-dir " + output_dir)
    shell_execute(command, os.path.join(output_dir, "txt.tmp"))


def i_create_empty_composite_and_add_source(step, add_sources=None,
                                            output_dir=None):
    """Step: I create empty BigML composite and then add list of sources
    and log results in <output_dir>
    """
    ok_(add_sources is not None and output_dir is not None)
    command = ("bigmler source --sources \"\" --add-sources " + add_sources +
               " --store --output-dir " + output_dir)
    shell_execute(command, os.path.join(output_dir, "txt.tmp"))


def remove_sources(step, output_dir=None):
    """Step: I remove the sources from a BigML composite and
    log results in <output_dir>
    """
    ok_(output_dir is not None)
    command = ("bigmler source --source " + world.source["resource"] +
               " --remove-sources " + ",".join(step.sources) +
               " --store --output-dir " + output_dir)
    shell_execute(command, os.path.join(output_dir, "txt.tmp"))


def delete_sources(step, output_dir=None):
    """Step: I delete the sources from a BigML composite and
    log results in <output_dir>
    """
    ok_(output_dir is not None)
    command = ("bigmler source --source " + world.source["resource"] +
               " --delete-sources " + ",".join(step.sources) +
               " --store --output-dir " + output_dir)
    shell_execute(command, os.path.join(output_dir, "txt.tmp"))


def i_create_annotated_source(
    step, annotations_file=None, images_file=None, output_dir=None):
    """Step: I create  BigML composite for an <annotations_file> and an
    <images_file> and log results in <output_dir>
    """
    ok_(annotations_file is not None and images_file is not None and
        output_dir is not None)
    annotations_file = res_filename(annotations_file)
    images_file = res_filename(images_file)
    command = ("bigmler source --annotations-file " + annotations_file +
               " --data " + images_file +
               " --store --output-dir " + output_dir)
    shell_execute(command, os.path.join(output_dir, "txt.tmp"))


def i_create_lang_annotated_source(
    step, annotations_dir=None, images_dir=None,
    annotations_language=None, output_dir=None):
    """Step: I create  BigML composite for a "<annotations_language>"
    <annotations_dir> and an <images_dir> and log results in <output_dir>
    """
    ok_(annotations_dir is not None and images_dir is not None and
        annotations_language is not None and output_dir is not None)
    command = ("bigmler source --annotations-dir " + annotations_dir +
               " --data " + images_dir + " --annotations-language " +
               annotations_language +
               " --store --output-dir " + output_dir)
    shell_execute(command, os.path.join(output_dir, "txt.tmp"))


def check_sources_in_composite(step):
    """Step: I check that the sources are in the composite"""
    eq_(step.sources, ["source/%s" % source for source in
                       world.source["object"].get("sources", [])])


def i_check_create_composite(step):
    """Step: I check that the composite has been created"""
    source_file = os.path.join(world.directory, "source")
    message = None
    try:
        last_source = read_sources(source_file)
        source = check_resource(
            last_source.strip(), world.api.get_source)
        if source['resource'] not in world.composites:
            world.composites.append(source['resource'])
        world.source = source
    except Exception as exc:
        message = str(exc)
    ok_(message is None, msg=message)


def check_images_number_in_composite(step, zip_file):
    """Step: I check the number of sources in the composite is
    <images_number>
    """
    sources = []
    zip_file = res_filename(zip_file)
    if os.path.isdir(zip_file):
        sources = filter_by_extension(glob.glob(os.path.join(zip_file, "**"),
                                                recursive=True),
                                      IMAGE_EXTENSIONS)
    else:
        with zipfile.ZipFile(zip_file) as zip_handler:
            sources = filter_by_extension(zip_handler.namelist(),
                IMAGE_EXTENSIONS)
    eq_(len(sources), len(step.sources), msg="Found %s sources in composite."
        " %s expected" % (len(step.sources), len(sources)))


def check_annotation_fields(step, annotations_file):
    """Checks the created label fields"""
    annotations_file = res_filename(annotations_file)
    fields = fields_from_annotations(annotations_file)
    field_labels = labels_from_annotations(annotations_file)
    source_fields = Fields(world.source)
    field_names = list(source_fields.fields_by_name.keys())
    for field in fields:
        ok_(field["name"] in field_names)
        field_labels[field["name"]].sort()
        source_fields.fields[
            source_fields.fields_by_name[
            field["name"]]]["values"].sort()
        eq_(field_labels[field["name"]],
            source_fields.fields[
                source_fields.fields_by_name[
                field["name"]]]["values"])


def i_extract_features(step, extracted_features, output_dir):
    """Updating extracted features from source"""
    extracted_features_str = " --".join(extracted_features)
    if extracted_features_str:
        extracted_features_str = " --" + extracted_features_str
    command = ("bigmler source --source " + world.source["resource"] + " " +
               extracted_features_str +
               " --store --output-dir " + output_dir)
    shell_execute(command, os.path.join(output_dir, "txt.tmp"))


def i_check_extracted_features(step, extracted_features):
    """Checking extracted features"""
    message = None
    try:
        extracted = world.source["object"]["image_analysis"][
            "extracted_features"]
        extracted = [TRANSLATED_FEATURES.get(feature, feature) for
            feature in extracted]
        ok_(all(feature in extracted for feature in extracted_features))
    except KeyError as exc:
        message = "Feature not found: %s" % str(exc)
    ok_(message is None, msg=message)


def i_extract_t2_features(step, attr, attr_value, output_dir):
    """Extracting features that involve wavelet subbands or cnns"""
    command = ("bigmler source --source " + world.source["resource"] +
               " --" +  attr + " " + str(attr_value) +
               " --store --output-dir " + output_dir)
    shell_execute(command, os.path.join(output_dir, "txt.tmp"))


def i_check_extracted_t2_features(step, attr, attr_value):
    """Checking wavelet extracted features"""
    attr = attr.replace("ws-level", "wavelet_subbands").replace("-", "_")
    message = None
    try:
        value = None
        extracted = world.source["object"]["image_analysis"][
            "extracted_features"]
        for feature in extracted:
            if isinstance(feature, list) and feature[0] == attr:
                value = feature[1]
        eq_(value, attr_value)
    except KeyError as exc:
        message = "Feature not found: %s" % str(exc)
    ok_(message is None, msg=message)



# -*- coding: utf-8 -*-
#
# Copyright 2014-2023 BigML
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

"""BigMLer - delete processing dispatching

"""


import sys
import os
import datetime
import shutil

import bigml.api

from bigml.api import get_resource_type
from bigml.constants import RENAMED_RESOURCES

import bigmler.utils as u
import bigmler.processing.args as a

from bigmler.defaults import DEFAULTS_FILE
from bigmler.command import get_stored_command, command_handling
from bigmler.dispatcher import SESSIONS_LOG, clear_log_files

COMMAND_LOG = ".bigmler_delete"
DIRS_LOG = ".bigmler_delete_dir_stack"
LOG_FILES = [COMMAND_LOG, DIRS_LOG, u.NEW_DIRS_LOG]
ROWS_LIMIT = 15
INDENT_IDS = 26
RESOURCES_LOG_FILES = set(['project', 'source', 'dataset', 'dataset_train',
                           'dataset_test', 'dataset_gen', 'dataset_cluster',
                           'dataset_parts', 'dataset_multi', 'models',
                           'ensembles', 'evaluations', 'optimls',
                           'clusters', 'batch_prediction', 'batch_centroid',
                           'anomalies', 'batch_anomaly_score', 'sample',
                           'associations', 'time_series', "deepnets",
                           'fusions', 'pcas', 'batch_projection',
                           'linear_regressions', 'logistic_regressions',
                           'scripts', 'library', 'execution',
                           'external_connector'])
STATUS_CODES = {
    "finished": bigml.api.FINISHED,
    "faulty": bigml.api.FAULTY,
    "waiting": bigml.api.WAITING,
    "queued": bigml.api.QUEUED,
    "started": bigml.api.STARTED,
    "in progress": bigml.api.IN_PROGRESS,
    "summarized": bigml.api.SUMMARIZED,
    "uploading": bigml.api.UPLOADING,
    "unknown": bigml.api.UNKNOWN,
    "runnable": bigml.api.RUNNABLE
}

GROUP_RESOURCES = ["project", "execution"]
SYNCHRONOUS_RESOURCES = ["project", "prediction", "centroid", "anomalyscore",
    "topicdistribution", "associationset", "forecast", "projection"]
TRASH_BIN = "Trash bin"

ALL_RESOURCE_TYPES = [
    "project", "execution", "script", "library", "optiml", "fusion",
    "batchprediction", "batchcentroid", "batchanomalyscore",
    "batchtopicdistribution", "batchprojection", "prediction", "anomalyscore",
    "associationset", "centroid", "forecast", "projection",
    "topicdistribution", "cluster", "ensemble", "model", "anomaly",
    "sample", "association", "logisticregression", "topicmodel",
    "timeseries", "deepnet", "pca", "linearregression", "externalconnector",
    "evaluation", "dataset", "source"]

SELECTOR_FILTERS = {
    "fusion": "fusion_status=false",
    "model": "ensemble=false;optiml_status=false;fusion_status=false",
    "ensemble": "optiml_status=false;fusion_status=false",
    "logisticregression": "optiml_status=false;fusion_status=false",
    "linearregression": "optiml_status=false;fusion_status=false",
    "deepnet": "optiml_status=false;fusion_status=false",
    "dataset": "cluster_status=false"}


def to_new_project(api, project_name, resource_ids):
    """Creates a new project and updates the resources to link them to the
    project. If the resources are projects, then prepends the name to the
    original name.
    """
    project_ids = []
    non_project_ids = []
    for resource_id in resource_ids:
        if resource_id.startswith("project/"):
            project_ids.append(resource_id)
        else:
            non_project_ids.append(resource_id)
    for project_id in project_ids:
        project = api.get_project(project_id)
        api.update_project(project_id, {"name": "%s: %s" % (project_name,
            project.get("object", {}).get("name"))})
    new_project_id = None
    if non_project_ids:
        new_project = api.create_project({"name": project_name})
        new_project_id = new_project["resource"]
    for resource_id in non_project_ids:
        resource_type = get_resource_type(resource_id)
        api.updaters[resource_type](resource_id, {"project": new_project_id})


def retrieve_resources(directory):
    """Searches recusively the user-given directory for resource log files
       and returns its ids.

    """
    log_ids = []
    if os.path.isdir(directory):
        for root, _, files in os.walk(directory):
            for resources_file in files:
                if resources_file in RESOURCES_LOG_FILES:
                    with open(os.path.join(root, resources_file)) as reader:
                        for line in reader:
                            resource_id = bigml.api.get_resource_id(
                                line.strip())
                            if resource_id is not None:
                                log_ids.append(resource_id)
    return list(set(log_ids))


def time_interval_qs(args, api):
    """Building the query string from the time interval user parameters.

    """
    query_string_list = []
    time_intervals = [(args.older_than, "created__lt=%s"),
                      (args.newer_than, "created__gt=%s")]
    for time_interval, expression in time_intervals:
        if time_interval:
            date_str = get_date(time_interval, api)
            if date_str:
                query_string_list.append(expression % date_str)
            else:
                sys.exit("The --older-than and --newer-than flags only accept "
                         "integers (number of days), dates in YYYY-MM-DD "
                         " format and resource ids."
                         " Please, double-check your input.")
    return query_string_list


def filter_qs(args):
    """Building the query string from the filter.

    """
    query_string_list = []
    if args.filter:
        query_string_list.append(args.filter)
    return query_string_list


def get_delete_list(args, api, query_list):
    """Building the list of resources to be deleted by adding the tag
       filtering user options to the
       previous ones for all the filtered resource types.

    """
    resource_selectors = filtered_selectors(args, api)
    delete_list = []

    if resource_selectors:
        for res_type, selector, api_call, filter_linked in resource_selectors:
            query_value = args.all_tag
            type_query_list = query_list[:]
            if args.all_tag or selector:
                if selector:
                    query_value = selector
                type_query_list.append("tags__in=%s" % query_value)
            if type_query_list and filter_linked:
                type_query_list.append(filter_linked)
            if type_query_list:
                status_code = STATUS_CODES[args.status]
                if res_type in SYNCHRONOUS_RESOURCES:
                    status_code = None
                delete_list.extend(u.list_ids(api_call,
                                              ";".join(type_query_list),
                                              status_code=status_code))
    return delete_list


def get_date(reference, api):
    """Extract the date from a given reference in days from now, date format
       or existing resource

    """
    days = None
    date = None
    try:
        days = int(reference)
        date = datetime.datetime.now() - datetime.timedelta(days=days)
    except ValueError:
        try:
            date = datetime.datetime.strptime(reference, '%Y-%m-%d')
            date = date.strftime('%Y-%m-%dT%H:%M:%S.%f')
        except ValueError:
            try:
                resource_type = bigml.api.get_resource_type(reference)
                resource = bigml.api.check_resource(reference,
                                                    api.getters[resource_type])
                date = resource['object']['created']
            except (TypeError, KeyError):
                return None
    return date


def resources_by_type(resources_list):
    """Sorts resources by type and summarizes their number """
    type_summary = {}
    sorted_resources = resources_list[:]
    sorted_resources.sort()
    for resource in resources_list:
        resource_type = bigml.api.get_resource_type(resource)
        if not resource_type in type_summary:
            type_summary[resource_type] = 0
        type_summary[resource_type] += 1
    return type_summary, sorted_resources


def filter_resource_types(delete_list, resource_types):
    """Filters the ids using the user-given resource types to ensure that
       only those resources will be deleted.

    """
    if resource_types is not None and resource_types:
        delete_list = [resource for resource in delete_list if
                       bigml.api.get_resource_type(resource) in resource_types]
    return delete_list


def filtered_selectors(args, api):
    """Returns the selectors for the user-given resource types to ensure that
       only those resources will be deleted.

    """

    resource_selectors = [
        (resource_type, getattr(args, "%s_tag" % RENAMED_RESOURCES.get(
         resource_type, resource_type)),
         api.listers[resource_type], SELECTOR_FILTERS.get(resource_type)) for
        resource_type in ALL_RESOURCE_TYPES]

    if args.all_tag is None and any(resource[1] is not None for resource in
                                     resource_selectors):
        # choose which selectors by tag are used and keep only these
        # selected by resource_tag
        resource_selectors = [resource for resource in resource_selectors if
                              resource[1] is not None]

    # selected by resource_types (on top of the tag selectors)
    selectors = [resource for resource in resource_selectors
                 if args.resource_types_ is None or
                 resource[0] in args.resource_types_]
    return selectors


#pylint: disable=locally-disabled,dangerous-default-value
def delete_dispatcher(args=sys.argv[1:]):
    """Parses command line and calls the different processing functions

    """

    command = command_handling(args, COMMAND_LOG)

    # Parses command line arguments.
    command_args = a.parse_and_check(command)
    if command_args.resume:
        command_args, session_file, _ = get_stored_command(
            args, command_args.debug, command_log=COMMAND_LOG,
            dirs_log=DIRS_LOG, sessions_log=SESSIONS_LOG)
    else:
        if command_args.output_dir is None:
            command_args.output_dir = a.OUTPUT_DIR
        directory = u.check_dir(os.path.join(command_args.output_dir, "tmp"))
        session_file = os.path.join(directory, SESSIONS_LOG)
        u.log_message(command.command + "\n", log_file=session_file)
        try:
            shutil.copy(DEFAULTS_FILE, os.path.join(directory, DEFAULTS_FILE))
        except IOError:
            pass
        u.sys_log_message("%s\n" % os.path.abspath(directory),
                          log_file=DIRS_LOG)

    # If --clear-logs the log files are cleared
    if "--clear-logs" in args:
        clear_log_files(LOG_FILES)

    # Creates the corresponding api instance
    api = a.get_api_instance(command_args, u.check_dir(session_file))

    delete_resources(command_args, api)
    u.log_message("_" * 80 + "\n", log_file=session_file)


def delete_resources(command_args, api, deleted_list=None, step=0):
    """Deletes the resources selected by the user given options

    """
    if deleted_list is None:
        deleted_list = []

    # deletion will be done in 3 steps. First projects and executions, then
    # composed resources (ensembles, etc.) and finally the rest of resources
    if step == 0:
        types = GROUP_RESOURCES
    elif step == 1:
        types = u.COMPOSED_RESOURCES
    else:
        types = [res_type for res_type in ALL_RESOURCE_TYPES if
                 res_type not in GROUP_RESOURCES and
                 res_type not in u.COMPOSED_RESOURCES]
    if command_args.output_dir is None:
        path = a.OUTPUT_DIR
    else:
        path = command_args.output_dir
    session_file = os.path.join(path, SESSIONS_LOG)
    message = u.dated("Retrieving objects to delete.\n")
    u.log_message(message, log_file=session_file,
                  console=command_args.verbosity)
    # Parses resource types to filter
    if command_args.resource_types is not None:
        resource_types = [resource_type.strip() for resource_type in
                          command_args.resource_types.split(',')]
        if command_args.exclude_types:
            # if that flag is set, the given resource_types are excluded
            command_args.resource_types_ = [
                resource_type for resource_type in types
                if command_args.resource_types is None or
                resource_type not in resource_types]
        else:
            command_args.resource_types_ = [
                res_type for res_type in resource_types if res_type in types]
    else:
        command_args.resource_types_ = types

    if command_args.resource_types_:
        delete_list = []
        # by ids
        if command_args.delete_list:
            message = u.dated("Resources selected from a user-given list of ids."
                              "\n\n")
            u.log_message(message, log_file=session_file,
                          console=command_args.verbosity)
            delete_list = [resource_id.strip() for resource_id in
                           command_args.delete_list.split(',')]
        # in file
        if command_args.delete_file:
            message = u.dated("Resources selected from user-given file.\n\n")
            u.log_message(message, log_file=session_file,
                          console=command_args.verbosity)
            if not os.path.exists(command_args.delete_file):
                sys.exit("File %s not found" % command_args.delete_file)
            with open(command_args.delete_file, "r") as delete_file:
                resource_id = bigml.api.get_resource_id(
                    delete_file.readline().strip())
                if resource_id:
                    delete_list.append(resource_id)
        # from directory
        if command_args.from_dir:
            message = u.dated("Resources extracted from directory logs.\n\n")
            u.log_message(message, log_file=session_file,
                          console=command_args.verbosity)

            delete_list.extend(retrieve_resources(command_args.from_dir))

        # by time interval and tag (plus filtered resource_types)
        qs_list = time_interval_qs(command_args, api)

        # by filter expression (plus filtered resource_types)
        filter_qs_list = filter_qs(command_args)

        qs_list.extend(filter_qs_list)
        if qs_list:
            message = u.dated("Resources filtered by expression:\n    %s\n\n" %
                ";".join(qs_list))
            u.log_message(message, log_file=session_file,
                          console=command_args.verbosity)

        delete_list.extend(get_delete_list(command_args, api, qs_list))

        # filter resource_types if any
        delete_list = filter_resource_types(delete_list,
                                            command_args.resource_types_)

        delete_list = [resource_id for resource_id in delete_list \
            if resource_id not in deleted_list]


        # if there are projects or executions, delete them first
        bulk_deletion = not command_args.bin and not command_args.dry_run and \
            any(resource_id.startswith("project/") or \
            (not command_args.execution_only and \
             resource_id.startswith("execution/")) for resource_id in delete_list)
        aprox = "*" if bulk_deletion else ""
        # ensure uniqueness
        delete_list = list(set(delete_list))
        types_summary, delete_list = resources_by_type( \
            delete_list)

        if types_summary.keys():
            message = ("Found resources with the following types: %s.\n"
                       % ",".join(types_summary.keys()))
            u.log_message(message, log_file=session_file,
                          console=command_args.verbosity)

            action_text = "Moving to Trash bin project" if command_args.bin else \
                "Deleting"
            action_text = "Dry-run for deleting" if command_args.dry_run else \
                action_text
            message = u.dated("%s %s objects%s.\n" % (action_text,
                len(delete_list), aprox))
            u.log_message(message, log_file=session_file,
                          console=command_args.verbosity)
            for resource_type, instances in list(types_summary.items()):
                message = "%s%ss: %s\n" % (" " * INDENT_IDS, resource_type,
                                           instances)
                u.log_message(message, log_file=session_file,
                              console=command_args.verbosity)
            if aprox != "":
                message = ("* WARNING: Deleting a project or an execution will delete"
                           " also its associated resources. Note that their IDs"
                           " may not be listed in this report.\n")
                u.log_message(message, log_file=session_file,
                              console=command_args.verbosity)
            if len(delete_list) > ROWS_LIMIT:
                pre_indent = INDENT_IDS - 4
                message = ("\n%s%s\n" % ((" " * pre_indent),
                                         ("Showing only the first %s resources.\n%s"
                                          "See details in bigmler_sessions"
                                          " file.\n") % (ROWS_LIMIT,
                                                         " " * pre_indent)))
                u.log_message(message, log_file=None,
                              console=command_args.verbosity)
            # Partial console message. Limited number of rows
            segment = delete_list[0: ROWS_LIMIT]
            message = ("\n%s" % (" " * INDENT_IDS)).join(segment)
            message = ("%s" % (" " * INDENT_IDS)) + message + "\n"
            u.log_message(message, log_file=None,
                          console=command_args.verbosity)
            # Complete message in session file
            message = ("\n%s" % (" " * INDENT_IDS)).join(delete_list)
            message = ("%s" % (" " * INDENT_IDS)) + message + "\n"
            u.log_message(message, log_file=session_file)
            if command_args.bin:
                to_new_project(api, TRASH_BIN, delete_list)
            elif not command_args.dry_run:
                command_args.qs = '' if not hasattr(command_args, "qs") else \
                    command_args.qs
                message = "Deleting...\n"
                u.log_message(message, log_file=session_file)
                u.delete(
                    api, delete_list,
                    exe_outputs=not command_args.execution_only,
                    query_string=command_args.qs)
        else:
            message = ("No resources found with the following types: %s.\n"
                       % ",".join(command_args.resource_types_))
            u.log_message(message, log_file=session_file,
                          console=command_args.verbosity)
        if step < 2:
            message = "Deleting...\n"
            u.log_message(message, log_file=session_file)
            # if projects and executions have already been deleted,
            # delete the rest
            deleted_list.extend(delete_list)
            delete_resources(command_args, api, deleted_list=deleted_list,
                             step=step + 1)

    if step == 2 and deleted_list:
        u.print_generated_files(path, log_file=session_file,
                                verbosity=command_args.verbosity)
